@@ -3,8 +3,6 @@
 Permission classes for the Consent API.
 """
 
-from __future__ import absolute_import, unicode_literals
-
 from rest_framework import permissions
 
 from enterprise.utils import get_request_value
@@ -16,8 +14,7 @@ class IsUserInRequest(permissions.BasePermission):
     """
 
     def has_permission(self, request, view):
-        if request.user.username == get_request_value(request, 'username', ''):
-            return True
+        return request.user.username == get_request_value(request, 'username', '')
 
 
 class IsStaffOrUserInRequest(IsUserInRequest):
@@ -30,4 +27,4 @@ class IsStaffOrUserInRequest(IsUserInRequest):
         if request.user.is_staff:
             return True
 
-        return super(IsStaffOrUserInRequest, self).has_permission(request, view)
+        return super().has_permission(request, view)
