@@ -3,13 +3,14 @@
 Models for edX Enterprise's Consent application.
 """
 
-from __future__ import absolute_import, unicode_literals
+import logging
 
 import six
 from opaque_keys import InvalidKeyError
 from opaque_keys.edx.keys import CourseKey
 from simple_history.models import HistoricalRecords
 
+from django.core.exceptions import ImproperlyConfigured
 from django.db import models
 from django.utils.encoding import python_2_unicode_compatible
 from django.utils.functional import lazy
@@ -20,10 +21,11 @@ from model_utils.models import TimeStampedModel
 
 from consent.errors import InvalidProxyConsent
 from consent.mixins import ConsentModelMixin
+from enterprise.api_client.discovery import get_course_catalog_api_service_client
 from enterprise.models import EnterpriseCustomer
-from enterprise.utils import parse_course_key
 
 mark_safe_lazy = lazy(mark_safe, six.text_type)  # pylint: disable=invalid-name
+LOGGER = logging.getLogger(__name__)
 
 
 class DataSharingConsentQuerySet(models.query.QuerySet):
@@ -42,6 +44,7 @@ class DataSharingConsentQuerySet(models.query.QuerySet):
         This customizes the queryset to return an instance of ``ProxyDataSharingConsent`` when
         the searched-for ``DataSharingConsent`` instance does not exist.
         """
+        # TODO: ENT-2010
         original_kwargs = kwargs.copy()
         if 'course_id' in kwargs:
             try:
@@ -58,7 +61,18 @@ class DataSharingConsentQuerySet(models.query.QuerySet):
                 except DataSharingConsent.DoesNotExist:
                     # A record for the course run didn't exist, so modify the query
                     # parameters to look for just a course record on the second pass.
-                    kwargs['course_id'] = parse_course_key(course_run_key)
+
+                    site = None
+                    if 'enterprise_customer' in kwargs:
+                        site = kwargs['enterprise_customer'].site
+
+                    try:
+                        course_id = get_course_catalog_api_service_client(site=site).get_course_id(
+                            course_identifier=course_run_key
+                        )
+                        kwargs['course_id'] = course_id
+                    except ImproperlyConfigured:
+                        LOGGER.warning('CourseCatalogApiServiceClient is improperly configured.')
 
         try:
             return self.get(*args, **kwargs)
@@ -74,8 +88,6 @@ class DataSharingConsentManager(models.Manager.from_queryset(DataSharingConsentQ
     ``DataSharingConsent`` object does not exist. Otherwise behaves the same as a normal manager.
     """
 
-    pass
-
 
 class ProxyDataSharingConsent(ConsentModelMixin):
     """
@@ -132,8 +144,8 @@ class ProxyDataSharingConsent(ConsentModelMixin):
             return None
         granted = all((child.granted for child in children))
         exists = any((child.exists for child in children))
-        usernames = set([child.username for child in children])
-        enterprises = set([child.enterprise_customer for child in children])
+        usernames = set(child.username for child in children)
+        enterprises = set(child.enterprise_customer for child in children)
         if not len(usernames) == len(enterprises) == 1:
             raise InvalidProxyConsent(
                 'Children used to create a bulk proxy consent object must '
@@ -219,12 +231,16 @@ class Consent(TimeStampedModel):
         return bool(self.pk)
 
 
-class DataSharingConsent(ConsentModelMixin, Consent):  # pylint: disable=model-missing-unicode
+class DataSharingConsent(ConsentModelMixin, Consent):  # pylint: disable=model-no-explicit-unicode
     """
     An abstract representation of Data Sharing Consent granted to an Enterprise for a course by a User.
 
     The model is used to store a persistent, historical consent state for users granting, not granting, or revoking
     data sharing consent to an Enterprise for a course.
+
+    .. pii: The username field inherited from Consent contains PII.
+    .. pii_types: username
+    .. pii_retirement: consumer_api
     """
 
     class Meta(Consent.Meta):
@@ -251,9 +267,11 @@ class DataSharingConsent(ConsentModelMixin, Consent):  # pylint: disable=model-m
 class DataSharingConsentTextOverrides(TimeStampedModel):
     """
     Stores texts overrides for data sharing consent page.
+
+    .. no_pii:
     """
 
-    class Meta(object):
+    class Meta:
         app_label = 'consent'
         verbose_name_plural = _('Data sharing consent text overrides')
 
@@ -382,9 +400,7 @@ class DataSharingConsentTextOverrides(TimeStampedModel):
     )
     published = models.BooleanField(
         default=False,
-        help_text=_(
-            _("Specifies whether data sharing consent page is published.")
-        )
+        help_text=_("Specifies whether data sharing consent page is published.")
     )
 
     def __str__(self):
