@@ -2,45 +2,41 @@
 """
 Forms to be used in the enterprise djangoapp.
 """
-from __future__ import absolute_import, unicode_literals
 
+import re
 from logging import getLogger
 
-from edx_rest_api_client.exceptions import HttpClientError, HttpServerError
+from edx_rbac.admin.forms import UserRoleAssignmentAdminForm
 
 from django import forms
 from django.conf import settings
-from django.contrib.auth.models import User
-from django.core.exceptions import ObjectDoesNotExist, ValidationError
-from django.core.urlresolvers import reverse
+from django.contrib import auth
+from django.core.exceptions import ValidationError
 from django.db.models.fields import BLANK_CHOICE_DASH
+from django.urls import reverse
 from django.utils.safestring import mark_safe
 from django.utils.translation import ugettext as _
 
 from enterprise import utils
-from enterprise.admin.utils import (
-    ProgramStatuses,
-    ValidationMessages,
-    email_or_username__to__email,
-    get_course_runs_from_program,
-    split_usernames_and_emails,
-    validate_email_to_link,
-)
-from enterprise.api_client.discovery import CourseCatalogApiClient
-from enterprise.api_client.lms import EnrollmentApiClient
+from enterprise.admin.utils import email_or_username__to__email, split_usernames_and_emails
+from enterprise.admin.widgets import SubmitInput
 from enterprise.models import (
     EnterpriseCustomer,
+    EnterpriseCustomerCatalog,
     EnterpriseCustomerIdentityProvider,
     EnterpriseCustomerReportingConfiguration,
+    EnterpriseFeatureUserRoleAssignment,
+    SystemWideEnterpriseUserRoleAssignment,
 )
-from enterprise.utils import MultipleProgramMatchError
+from enterprise.utils import ValidationMessages, validate_email_to_link
 
 try:
-    from third_party_auth.models import SAMLProviderConfig as saml_provider_configuration
+    from common.djangoapps.third_party_auth.models import SAMLProviderConfig as saml_provider_configuration
 except ImportError:
     saml_provider_configuration = None
 
 logger = getLogger(__name__)  # pylint: disable=invalid-name
+User = auth.get_user_model()
 
 
 class ManageLearnersForm(forms.Form):
@@ -59,18 +55,15 @@ class ManageLearnersForm(forms.Form):
         required=False,
         help_text=_(
             "The .csv file must have a column of email addresses, indicated "
-            "by the heading 'email' in the first row."
+            "by the heading 'email' in the first row. Optionally, the .csv file may contain "
+            "a column of course run keys, indicated by the heading 'course_id' in the first row, to "
+            "enroll learners in multiple courses."
         )
     )
     course = forms.CharField(
         label=_("Enroll these learners in this course"), required=False,
         help_text=_("To enroll learners in a course, enter a course ID."),
     )
-    program = forms.CharField(
-        label=_("Or enroll these learners in this program"), required=False,
-        help_text=_(
-            "To enroll learners in a program, enter a program name or ID.")
-    )
     course_mode = forms.ChoiceField(
         label=_("Course enrollment track"), required=False,
         choices=BLANK_CHOICE_DASH + [
@@ -82,8 +75,17 @@ class ManageLearnersForm(forms.Form):
             ("honor", _("Honor")),
         ],
     )
+    reason = forms.CharField(label=_("Reason for manual enrollment"), required=False)
+    sales_force_id = forms.CharField(label=_("Salesforce Opportunity ID"), required=False)
+    discount = forms.DecimalField(
+        label=_("Discount percentage for manual enrollment"),
+        help_text=_("Discount percentage should be from 0 to 100"),
+        required=True,
+        decimal_places=5,
+        initial=0.0
+    )
 
-    class NotificationTypes(object):
+    class NotificationTypes:
         """
         Namespace class for notification types
         """
@@ -101,14 +103,14 @@ class ManageLearnersForm(forms.Form):
         required=False,
     )
 
-    class Modes(object):
+    class Modes:
         """
         Namespace class for form modes.
         """
         MODE_SINGULAR = "singular"
         MODE_BULK = "bulk"
 
-    class Fields(object):
+    class Fields:
         """
         Namespace class for field names.
         """
@@ -119,14 +121,17 @@ class ManageLearnersForm(forms.Form):
         MODE = "mode"
         COURSE = "course"
         COURSE_MODE = "course_mode"
-        PROGRAM = "program"
         NOTIFY = "notify_on_enrollment"
+        REASON = "reason"
+        SALES_FORCE_ID = "sales_force_id"
+        DISCOUNT = "discount"
 
-    class CsvColumns(object):
+    class CsvColumns:
         """
         Namespace class for CSV column names.
         """
         EMAIL = "email"
+        COURSE_ID = "course_id"
 
     def __init__(self, *args, **kwargs):
         """
@@ -140,7 +145,7 @@ class ManageLearnersForm(forms.Form):
         user = kwargs.pop('user', None)
         self._user = user
         self._enterprise_customer = kwargs.pop('enterprise_customer', None)
-        super(ManageLearnersForm, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
 
     def clean_email_or_username(self):
         """
@@ -161,21 +166,32 @@ class ManageLearnersForm(forms.Form):
             for email in split_usernames_and_emails(email):
                 validate_email_to_link(
                     email,
+                    self._enterprise_customer,
                     None,
                     ValidationMessages.INVALID_EMAIL_OR_USERNAME,
-                    ignore_existing=True
+                    raise_exception=False,
                 )
             email = email_or_username
         else:
             validate_email_to_link(
                 email,
+                self._enterprise_customer,
                 email_or_username,
                 ValidationMessages.INVALID_EMAIL_OR_USERNAME,
-                ignore_existing=True
+                raise_exception=False,
             )
 
         return email
 
+    def clean_discount(self):
+        """
+        Verify that discount value should be from 0 to 100.
+        """
+        discount = self.cleaned_data[self.Fields.DISCOUNT]
+        if discount < 0.0 or discount > 100.0:
+            raise ValidationError(ValidationMessages.INVALID_DISCOUNT)
+        return discount
+
     def clean_course(self):
         """
         Verify course ID and retrieve course details.
@@ -183,42 +199,15 @@ class ManageLearnersForm(forms.Form):
         course_id = self.cleaned_data[self.Fields.COURSE].strip()
         if not course_id:
             return None
-        try:
-            client = EnrollmentApiClient()
-            return client.get_course_details(course_id)
-        except (HttpClientError, HttpServerError):
-            raise ValidationError(ValidationMessages.INVALID_COURSE_ID.format(course_id=course_id))
+        enterprise_customer = utils.get_enterprise_customer(self._enterprise_customer.uuid)
+        course_details = utils.validate_course_exists_for_enterprise(enterprise_customer, course_id)
+        return course_details
 
-    def clean_program(self):
+    def clean_reason(self):
         """
-        Clean program.
-
-        Try obtaining program treating form value as program UUID or title.
-
-        Returns:
-            dict: Program information if program found
+        Clean the reason for enrollment field
         """
-        program_id = self.cleaned_data[self.Fields.PROGRAM].strip()
-        if not program_id:
-            return None
-
-        try:
-            client = CourseCatalogApiClient(self._user, self._enterprise_customer.site)
-            program = client.get_program_by_uuid(program_id) or client.get_program_by_title(program_id)
-        except MultipleProgramMatchError as exc:
-            raise ValidationError(ValidationMessages.MULTIPLE_PROGRAM_MATCH.format(program_count=exc.programs_matched))
-        except (HttpClientError, HttpServerError):
-            raise ValidationError(ValidationMessages.INVALID_PROGRAM_ID.format(program_id=program_id))
-
-        if not program:
-            raise ValidationError(ValidationMessages.INVALID_PROGRAM_ID.format(program_id=program_id))
-
-        if program['status'] != ProgramStatuses.ACTIVE:
-            raise ValidationError(
-                ValidationMessages.PROGRAM_IS_INACTIVE.format(program_id=program_id, status=program['status'])
-            )
-
-        return program
+        return self.cleaned_data.get(self.Fields.REASON).strip()
 
     def clean_notify(self):
         """
@@ -233,7 +222,7 @@ class ManageLearnersForm(forms.Form):
         In this case, the form can be used to link single user or bulk link multiple users. These are mutually
         exclusive modes, so this method checks that only one field is passed.
         """
-        cleaned_data = super(ManageLearnersForm, self).clean()
+        cleaned_data = super().clean()
 
         # Here we take values from `data` (and not `cleaned_data`) as we need raw values - field clean methods
         # might "invalidate" the value and set it to None, while all we care here is if it was provided at all or not
@@ -254,17 +243,33 @@ class ManageLearnersForm(forms.Form):
         cleaned_data[self.Fields.MODE] = mode
         cleaned_data[self.Fields.NOTIFY] = self.clean_notify()
 
+        self._validate_bulk_upload()
         self._validate_course()
-        self._validate_program()
-
-        if self.data.get(self.Fields.PROGRAM, None) and self.data.get(self.Fields.COURSE, None):
-            raise ValidationError(ValidationMessages.COURSE_AND_PROGRAM_ERROR)
+        self._validate_reason()
 
         return cleaned_data
 
+    def _validate_bulk_upload(self):
+        """
+        Verify that when a csv file contains a "course_id" column, there is not a singular course ID
+        specified. Also verify that a course mode is specified, which is applied to all enrollments.
+        """
+        bulk_upload_csv = self.cleaned_data.get(self.Fields.BULK_UPLOAD)
+        if bulk_upload_csv:
+            bulk_csv_contents = bulk_upload_csv.read()
+            csv_column_names = bulk_csv_contents.decode('utf-8').split('\n', 1)[0].split(',')
+            if self.CsvColumns.COURSE_ID in csv_column_names:
+                # course id column exists in csv, so validate conditionally required fields
+                if self.cleaned_data.get(self.Fields.COURSE):
+                    raise ValidationError(ValidationMessages.BOTH_COURSE_FIELDS_SPECIFIED)
+                if not self.cleaned_data.get(self.Fields.COURSE_MODE):
+                    raise ValidationError(ValidationMessages.COURSE_WITHOUT_COURSE_MODE)
+                if not self.cleaned_data.get(self.Fields.REASON):
+                    raise ValidationError(ValidationMessages.MISSING_REASON)
+
     def _validate_course(self):
         """
-        Verify that the selected mode is valid for the given course .
+        Verify that the selected mode is valid for the given course.
         """
         # Verify that the selected mode is valid for the given course .
         course_details = self.cleaned_data.get(self.Fields.COURSE)
@@ -280,114 +285,156 @@ class ManageLearnersForm(forms.Form):
                 ))
                 raise ValidationError({self.Fields.COURSE_MODE: error})
 
-    def _validate_program(self):
+    def _validate_reason(self):
         """
-        Verify that selected mode is available for program and all courses in the program
+        Verify that the reason field is populated if the new learner(s) is being enrolled
+        in a course or program.
         """
-        program = self.cleaned_data.get(self.Fields.PROGRAM)
-        if not program:
-            return
+        course = self.cleaned_data.get(self.Fields.COURSE)
+        reason = self.cleaned_data.get(self.Fields.REASON)
 
-        course_runs = get_course_runs_from_program(program)
-        try:
-            client = CourseCatalogApiClient(self._user, self._enterprise_customer.site)
-            available_modes = client.get_common_course_modes(course_runs)
-            course_mode = self.cleaned_data.get(self.Fields.COURSE_MODE)
-        except (HttpClientError, HttpServerError):
-            raise ValidationError(
-                ValidationMessages.FAILED_TO_OBTAIN_COURSE_MODES.format(program_title=program.get("title"))
-            )
+        if course and not reason:
+            raise ValidationError(ValidationMessages.MISSING_REASON)
 
-        if not course_mode:
-            raise ValidationError(ValidationMessages.COURSE_WITHOUT_COURSE_MODE)
-        if course_mode not in available_modes:
-            raise ValidationError(ValidationMessages.COURSE_MODE_NOT_AVAILABLE.format(
-                mode=course_mode, program_title=program.get("title"), modes=", ".join(available_modes)
-            ))
+
+class ManageLearnersDataSharingConsentForm(forms.Form):
+    """
+    Form to request DSC from a learner.
+    """
+    email_or_username = forms.CharField(
+        label=_("Email/Username"),
+        help_text=_("Enter an email address or username."),
+        required=True
+    )
+    course = forms.CharField(
+        label=_("Course"),
+        help_text=_("Enter the course run key."),
+        required=True,
+    )
+
+    class Fields:
+        """
+        Namespace class for field names.
+        """
+        EMAIL_OR_USERNAME = "email_or_username"
+        COURSE = "course"
+
+    def __init__(self, *args, **kwargs):
+        """
+        Initializes form with current enterprise customer.
+
+        Arguments:
+            enterprise_customer (enterprise.models.EnterpriseCustomer): current customer
+        """
+        self._enterprise_customer = kwargs.pop('enterprise_customer', None)
+        super().__init__(*args, **kwargs)
+
+    def clean_course(self):
+        """
+        Verify course ID has an associated course in LMS.
+        """
+        course_id = self.cleaned_data[self.Fields.COURSE].strip()
+        if not course_id:
+            return None
+        enterprise_customer = utils.get_enterprise_customer(self._enterprise_customer.uuid)
+        utils.validate_course_exists_for_enterprise(enterprise_customer, course_id)
+        return course_id
+
+    def clean_email_or_username(self):
+        """
+        Verify email_or_username has associated user in our database.
+        """
+        email_or_username = self.cleaned_data[self.Fields.EMAIL_OR_USERNAME].strip()
+        email = email_or_username__to__email(email_or_username)
+        # Check whether user exists in our database.
+        if not User.objects.filter(email=email).exists():
+            raise ValidationError(ValidationMessages.USER_NOT_EXIST.format(email=email))
+
+        # Check whether user in linked to the enterprise customer.
+        if not self.is_user_linked(email):
+            raise ValidationError(ValidationMessages.USER_NOT_LINKED)
+        return email
+
+    def is_course_in_catalog(self, course_id):
+        """
+        Check whether course exists in enterprise customer catalog.
+        """
+        enterprise_customer = utils.get_enterprise_customer(self._enterprise_customer.uuid)
+        return enterprise_customer.catalog_contains_course(course_id)
+
+    def is_user_linked(self, email):
+        """
+        Check whether user is linked to the enterprise customer or not.
+        """
+        user = User.objects.get(email=email)
+        return utils.get_enterprise_customer_user(user.id, self._enterprise_customer.uuid)
 
 
 class EnterpriseCustomerAdminForm(forms.ModelForm):
     """
     Alternate form for the EnterpriseCustomer admin page.
-
-    This form fetches catalog names and IDs from the course catalog API.
     """
     class Meta:
         model = EnterpriseCustomer
         fields = (
             "name",
+            "slug",
+            "country",
             "active",
+            "customer_type",
             "site",
-            "catalog",
             "enable_data_sharing_consent",
             "enforce_data_sharing_consent",
             "enable_audit_enrollment",
             "enable_audit_data_reporting",
             "replace_sensitive_sso_username",
             "hide_course_original_price",
+            "enable_portal_code_management_screen",
+            "enable_portal_subscription_management_screen",
+            "enable_learner_portal",
+            "enable_integrated_customer_learner_portal_search",
+            "enable_analytics_screen",
+            "enable_portal_reporting_config_screen",
+            "enable_portal_saml_configuration_screen",
+            "enable_portal_lms_configurations_screen",
+            "enable_slug_login",
+            "contact_email",
+            "default_contract_discount",
+            "default_language",
+            "sender_alias",
         )
 
-    class Media:
-        js = ('enterprise/admin/enterprise_customer.js', )
 
-    def __init__(self, *args, **kwargs):
+class EnterpriseCustomerCatalogAdminForm(forms.ModelForm):
+    """
+        form for EnterpriseCustomerCatalogAdmin class.
+    """
+    class Meta:
+        model = EnterpriseCustomerCatalog
+        fields = "__all__"
+
+    preview_button = forms.Field(required=False, label='Actions', widget=SubmitInput(attrs={'value': _('Preview')}),
+                                 help_text=_("Hold Ctrl when clicking on button to open Preview in new tab"))
+
+    @staticmethod
+    def get_catalog_preview_uuid(post_data):  # pylint: disable=invalid-name
         """
-        Initialize the form.
+        Return the uuid of the catalog the preview button was clicked on
+        There must be only one preview button in the POST data.
 
-        Substitute a ChoiceField in for the catalog field that would
-        normally be set up as a plain number entry field.
+        e.g: 'enterprise_customer_catalogs-0-preview_button'
         """
-        super(EnterpriseCustomerAdminForm, self).__init__(*args, **kwargs)
-
-        self.fields['catalog'] = forms.ChoiceField(
-            choices=self.get_catalog_options(),
-            required=False,
-            # pylint: disable=bad-continuation
-            help_text='<a id="catalog-details-link" href="#" target="_blank" '
-                      'data-url-template="{catalog_admin_change_url}"> View catalog details.</a>'
-                      ' <p style="margin-top:-4px;"><a href="{catalog_admin_add_url}"'
-                      ' target="_blank">Create a new catalog</a></p>'.format(
-                catalog_admin_change_url=utils.get_catalog_admin_url_template(mode='change'),
-                catalog_admin_add_url=utils.get_catalog_admin_url_template(mode='add'))
-        )
-
-    def get_catalog_options(self):
-        """
-        Retrieve a list of catalog ID and name pairs.
-
-        Once retrieved, these name pairs can be used directly as a value
-        for the `choices` argument to a ChoiceField.
-        """
-        # TODO: We will remove the discovery service catalog implementation
-        # once we have fully migrated customer's to EnterpriseCustomerCatalogs.
-        # For now, this code will prevent an admin from creating a new
-        # EnterpriseCustomer with a discovery service catalog. They will have to first
-        # save the EnterpriseCustomer admin form and then edit the EnterpriseCustomer
-        # to add a discovery service catalog.
-        if hasattr(self.instance, 'site'):
-            catalog_api = CourseCatalogApiClient(self.user, self.instance.site)
-        else:
-            catalog_api = CourseCatalogApiClient(self.user)
-        catalogs = catalog_api.get_all_catalogs()
-        # order catalogs by name.
-        catalogs = sorted(catalogs, key=lambda catalog: catalog.get('name', '').lower())
-
-        return BLANK_CHOICE_DASH + [
-            (catalog['id'], catalog['name'],)
-            for catalog in catalogs
-        ]
-
-    def clean(self):
-        """
-        Clean form fields prior to database entry.
-
-        In this case, the major cleaning operation is substituting a None value for a blank
-        value in the Catalog field.
-        """
-        cleaned_data = super(EnterpriseCustomerAdminForm, self).clean()
-        if 'catalog' in cleaned_data and not cleaned_data['catalog']:
-            cleaned_data['catalog'] = None
-        return cleaned_data
+        preview_button_expression = re.compile(r'enterprise_customer_catalogs-\d+-preview_button')
+        clicked_button_index_expression = re.compile(r'-(.+?)-')
+        count = 0
+        preview_button_index = None
+        for key, _ in post_data.items():
+            if preview_button_expression.match(key):
+                count += 1
+                preview_button_index = clicked_button_index_expression.search(key).group(1)
+        if count == 1:
+            return post_data.get('enterprise_customer_catalogs-' + preview_button_index + '-uuid')
+        return None
 
 
 class EnterpriseCustomerIdentityProviderAdminForm(forms.ModelForm):
@@ -407,7 +454,7 @@ class EnterpriseCustomerIdentityProviderAdminForm(forms.ModelForm):
 
         Substitutes CharField with TypedChoiceField for the provider_id field.
         """
-        super(EnterpriseCustomerIdentityProviderAdminForm, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
         idp_choices = utils.get_idp_choices()
         help_text = ''
         if saml_provider_configuration:
@@ -417,12 +464,15 @@ class EnterpriseCustomerIdentityProviderAdminForm(forms.ModelForm):
                 saml_provider_configuration._meta.model_name))
             if provider_id:
                 identity_provider = utils.get_identity_provider(provider_id)
-                update_url = url + '?source={}'.format(identity_provider.pk)
-                help_text = '<p><a href="{update_url}" target="_blank">View "{identity_provider}" details</a><p>'.\
-                    format(update_url=update_url, identity_provider=identity_provider.name)
-
-            help_text += '<p style="margin-top:-5px;"><a target="_blank" href={add_url}>' \
-                         'Create a new identity provider</a></p>'.format(add_url=url)
+                if identity_provider:
+                    update_url = url + '?source={}'.format(identity_provider.pk)
+                    help_text = '<p><a href="{update_url}" target="_blank">View "{identity_provider}" details</a><p>'.\
+                        format(update_url=update_url, identity_provider=identity_provider.name)
+                else:
+                    help_text += '<p style="margin-top:-5px;"> Make sure you have added a valid provider_id.</p>'
+            else:
+                help_text += '<p style="margin-top:-5px;"><a target="_blank" href={add_url}>' \
+                             'Create a new identity provider</a></p>'.format(add_url=url)
 
         if idp_choices is not None:
             self.fields['provider_id'] = forms.TypedChoiceField(
@@ -437,7 +487,7 @@ class EnterpriseCustomerIdentityProviderAdminForm(forms.ModelForm):
 
         1. Validate that selected site for enterprise customer matches with the selected identity provider's site.
         """
-        super(EnterpriseCustomerIdentityProviderAdminForm, self).clean()
+        super().clean()
 
         provider_id = self.cleaned_data.get('provider_id', None)
         enterprise_customer = self.cleaned_data.get('enterprise_customer', None)
@@ -446,9 +496,9 @@ class EnterpriseCustomerIdentityProviderAdminForm(forms.ModelForm):
             # field validation for either provider_id or enterprise_customer has already raised
             # a validation error.
             return
-        try:
-            identity_provider = utils.get_identity_provider(provider_id)
-        except ObjectDoesNotExist:
+
+        identity_provider = utils.get_identity_provider(provider_id)
+        if not identity_provider:
             # This should not happen, as identity providers displayed in drop down are fetched dynamically.
             message = _(
                 "The specified Identity Provider does not exist. For more "
@@ -474,6 +524,19 @@ class EnterpriseCustomerIdentityProviderAdminForm(forms.ModelForm):
                 ),
             )
 
+        # There should be always one default provider in case an enterprise has multiple providers.
+        identity_provider_total_forms = int(self.data.get('enterprise_customer_identity_providers-TOTAL_FORMS'))
+        default_provider_values = []
+        for form_num in range(identity_provider_total_forms):
+            default_provider_values.append(
+                self.data.get('enterprise_customer_identity_providers-{}-default_provider'.format(form_num))
+            )
+        providers_marked_default = default_provider_values.count('on')
+        if providers_marked_default > 1:
+            raise ValidationError('Please select only one default provider.')
+        if identity_provider_total_forms > 1 and providers_marked_default == 0:
+            raise ValidationError('Please select one default provider.')
+
 
 class EnterpriseCustomerReportingConfigAdminForm(forms.ModelForm):
     """
@@ -482,6 +545,11 @@ class EnterpriseCustomerReportingConfigAdminForm(forms.ModelForm):
     This form uses the PasswordInput widget to obscure passwords as they are
     being entered by the user.
     """
+    enterprise_customer_catalogs = forms.ModelMultipleChoiceField(
+        EnterpriseCustomerCatalog.objects.all(),
+        required=False,
+    )
+
     class Meta:
         model = EnterpriseCustomerReportingConfiguration
         fields = (
@@ -490,10 +558,12 @@ class EnterpriseCustomerReportingConfigAdminForm(forms.ModelForm):
             "data_type",
             "report_type",
             "delivery_method",
+            "pgp_encryption_key",
             "frequency",
             "day_of_month",
             "day_of_week",
             "hour_of_day",
+            "include_date",
             "email",
             "decrypted_password",
             "sftp_hostname",
@@ -501,12 +571,49 @@ class EnterpriseCustomerReportingConfigAdminForm(forms.ModelForm):
             "sftp_username",
             "decrypted_sftp_password",
             "sftp_file_path",
+            "enterprise_customer_catalogs",
         )
         widgets = {
             'decrypted_password': forms.widgets.PasswordInput(),
             'decrypted_sftp_password': forms.widgets.PasswordInput(),
         }
 
+    def clean(self):
+        """
+        Override of clean method to perform additional validation
+        """
+        cleaned_data = super().clean()
+        report_customer = cleaned_data.get('enterprise_customer')
+        data_type = cleaned_data.get('data_type')
+
+        if data_type in EnterpriseCustomerReportingConfiguration.PEARSON_ONLY_REPORTS \
+                and report_customer.name != 'Pearson':
+            message = _(
+                'This data_type "{data_type}" is not supported for enterprise'
+                'customer {enterprise_customer}. Please select a different data_type.',
+            ).format(
+                enterprise_customer=report_customer,
+                data_type=data_type,
+            )
+            self.add_error('data_type', message)
+
+        # Check that any selected catalogs are tied to the selected enterprise.
+        invalid_catalogs = [
+            '{} ({})'.format(catalog.title, catalog.uuid)
+            for catalog in cleaned_data.get('enterprise_customer_catalogs')
+            if catalog.enterprise_customer != report_customer
+        ]
+
+        if invalid_catalogs:
+            message = _(
+                'These catalogs for reporting do not match enterprise'
+                'customer {enterprise_customer}: {invalid_catalogs}',
+            ).format(
+                enterprise_customer=report_customer,
+                invalid_catalogs=invalid_catalogs,
+            )
+            self.add_error('enterprise_customer_catalogs', message)
+
 
 class TransmitEnterpriseCoursesForm(forms.Form):
     """
@@ -528,11 +635,50 @@ class TransmitEnterpriseCoursesForm(forms.Form):
 
         try:
             User.objects.get(username=channel_worker_username)
-        except User.DoesNotExist:
+        except User.DoesNotExist as error:
             raise ValidationError(
                 ValidationMessages.INVALID_CHANNEL_WORKER.format(
                     channel_worker_username=channel_worker_username
                 )
-            )
+            ) from error
 
         return channel_worker_username
+
+
+class SystemWideEnterpriseUserRoleAssignmentForm(UserRoleAssignmentAdminForm):
+    """
+    Form for SystemWideEnterpriseUserRoleAssignments.
+    """
+
+    class Meta:
+        model = SystemWideEnterpriseUserRoleAssignment
+        fields = [
+            'user',
+            'role',
+            'enterprise_customer',
+            'applies_to_all_contexts',
+        ]
+
+    def __init__(self, *args, **kwargs):
+        """
+        Modifies the enterprise_customer queryset to only include
+        EnterpriseCustomer instances to which the user for this instance
+        of a SystemWideEnterpriseUserRoleAssignment is linked.
+        """
+        super().__init__(*args, **kwargs)
+        try:
+            self.fields['enterprise_customer'].queryset = EnterpriseCustomer.objects.filter(
+                enterprise_customer_users__user_id=self.instance.user.id,
+            )
+        except SystemWideEnterpriseUserRoleAssignment.user.RelatedObjectDoesNotExist:
+            self.fields['enterprise_customer'].queryset = EnterpriseCustomer.objects.none()
+
+
+class EnterpriseFeatureUserRoleAssignmentForm(UserRoleAssignmentAdminForm):
+    """
+    Form for EnterpriseFeatureUserRoleAssignments.
+    """
+
+    class Meta:
+        model = EnterpriseFeatureUserRoleAssignment
+        fields = ['user', 'role']
