@@ -2,22 +2,32 @@
 """
 Serializers for enterprise api version 1.
 """
-from __future__ import absolute_import, unicode_literals
 
 import copy
+from logging import getLogger
 
 from edx_rest_api_client.exceptions import HttpClientError
 from rest_framework import serializers
 from rest_framework.settings import api_settings
 
-from django.contrib.auth.models import User
+from django.contrib import auth
 from django.contrib.sites.models import Site
 from django.utils.translation import ugettext_lazy as _
 
 from enterprise import models, utils
-from enterprise.api.v1.mixins import EnterpriseCourseContextSerializerMixin
+from enterprise.api.v1.fields import Base64EmailCSVField
 from enterprise.api_client.lms import ThirdPartyAuthApiClient
-from enterprise.utils import track_enrollment
+from enterprise.constants import ENTERPRISE_PERMISSION_GROUPS, DefaultColors
+from enterprise.utils import (
+    CourseEnrollmentDowngradeError,
+    CourseEnrollmentPermissionError,
+    get_last_course_run_end_date,
+    has_course_run_available_for_enrollment,
+    track_enrollment,
+)
+
+LOGGER = getLogger(__name__)
+User = auth.get_user_model()
 
 
 class ImmutableStateSerializer(serializers.Serializer):
@@ -29,13 +39,11 @@ class ImmutableStateSerializer(serializers.Serializer):
         """
         Do not perform any operations for state changing requests.
         """
-        pass
 
     def update(self, instance, validated_data):
         """
         Do not perform any operations for state changing requests.
         """
-        pass
 
 
 class ResponsePaginationSerializer(ImmutableStateSerializer):
@@ -81,20 +89,57 @@ class EnterpriseCustomerBrandingConfigurationSerializer(serializers.ModelSeriali
     class Meta:
         model = models.EnterpriseCustomerBrandingConfiguration
         fields = (
-            'enterprise_customer', 'logo'
+            'enterprise_customer',
+            'enterprise_slug',
+            'logo',
+            'primary_color',
+            'secondary_color',
+            'tertiary_color',
         )
 
+    enterprise_customer = serializers.SerializerMethodField()
+    enterprise_slug = serializers.SerializerMethodField()
+    logo = serializers.SerializerMethodField()
+    primary_color = serializers.SerializerMethodField()
+    secondary_color = serializers.SerializerMethodField()
+    tertiary_color = serializers.SerializerMethodField()
 
-class EnterpriseCustomerEntitlementSerializer(serializers.ModelSerializer):
-    """
-    Serializer for EnterpriseCustomerEntitlement model.
-    """
+    def get_enterprise_customer(self, obj):
+        """
+        Return a string representation of the associated enterprise customer's UUID.
+        """
+        return str(obj.enterprise_customer.uuid)
 
-    class Meta:
-        model = models.EnterpriseCustomerEntitlement
-        fields = (
-            'enterprise_customer', 'entitlement_id'
-        )
+    def get_enterprise_slug(self, obj):
+        """
+        Return the slug of the associated enterprise customer.
+        """
+        return obj.enterprise_customer.slug
+
+    def get_logo(self, obj):
+        """
+        Use EnterpriseCustomerBrandingConfiguration.safe_logo_url to return an absolute
+        URL for either the saved customer logo or the platform logo by default
+        """
+        return obj.safe_logo_url
+
+    def get_primary_color(self, obj):
+        """
+        Return the primary color of the branding config OR the default primary color code
+        """
+        return obj.primary_color if obj.primary_color else DefaultColors.PRIMARY
+
+    def get_secondary_color(self, obj):
+        """
+        Return the secondary color of the branding config OR the default secondary color code
+        """
+        return obj.secondary_color if obj.secondary_color else DefaultColors.SECONDARY
+
+    def get_tertiary_color(self, obj):
+        """
+        Return the tertiary color of the branding config OR the default tertiary color code
+        """
+        return obj.tertiary_color if obj.tertiary_color else DefaultColors.TERTIARY
 
 
 class EnterpriseCustomerSerializer(serializers.ModelSerializer):
@@ -105,16 +150,37 @@ class EnterpriseCustomerSerializer(serializers.ModelSerializer):
     class Meta:
         model = models.EnterpriseCustomer
         fields = (
-            'uuid', 'name', 'catalog', 'active', 'site', 'enable_data_sharing_consent', 'enforce_data_sharing_consent',
-            'branding_configuration', 'enterprise_customer_entitlements', 'identity_provider',
-            'enable_audit_enrollment', 'replace_sensitive_sso_username',
+            'uuid', 'name', 'slug', 'active', 'site', 'enable_data_sharing_consent',
+            'enforce_data_sharing_consent', 'branding_configuration',
+            'identity_provider', 'enable_audit_enrollment', 'replace_sensitive_sso_username',
+            'enable_portal_code_management_screen', 'sync_learner_profile_data', 'enable_audit_data_reporting',
+            'enable_learner_portal', 'enable_portal_reporting_config_screen',
+            'enable_portal_saml_configuration_screen', 'contact_email',
+            'enable_portal_subscription_management_screen', 'hide_course_original_price', 'enable_analytics_screen',
+            'enable_integrated_customer_learner_portal_search',
+            'enable_portal_lms_configurations_screen', 'sender_alias',
         )
 
     site = SiteSerializer()
-    branding_configuration = EnterpriseCustomerBrandingConfigurationSerializer()
-    enterprise_customer_entitlements = EnterpriseCustomerEntitlementSerializer(  # pylint: disable=invalid-name
-        many=True,
-    )
+    branding_configuration = serializers.SerializerMethodField()
+
+    def get_branding_configuration(self, obj):
+        """
+        Return the serialized branding configuration object OR default object if null
+        """
+        return EnterpriseCustomerBrandingConfigurationSerializer(obj.safe_branding_configuration).data
+
+
+class EnterpriseCustomerBasicSerializer(serializers.ModelSerializer):
+    """
+    Serializer for EnterpriseCustomer model only for name and id fields.
+    """
+
+    class Meta:
+        model = models.EnterpriseCustomer
+        fields = ('id', 'name')
+
+    id = serializers.CharField(source='uuid')  # pylint: disable=invalid-name
 
 
 class EnterpriseCourseEnrollmentReadOnlySerializer(serializers.ModelSerializer):
@@ -149,13 +215,18 @@ class EnterpriseCourseEnrollmentWriteSerializer(serializers.ModelSerializer):
         """
         try:
             user = User.objects.get(username=value)
-        except User.DoesNotExist:
-            raise serializers.ValidationError("User does not exist")
+        except User.DoesNotExist as no_user_error:
+            error_message = ('[Enterprise API] The username for creating an EnterpriseCourseEnrollment'
+                             ' record does not exist. User: {}').format(value)
+            LOGGER.error(error_message)
+            raise serializers.ValidationError("User does not exist") from no_user_error
 
         try:
             enterprise_customer_user = models.EnterpriseCustomerUser.objects.get(user_id=user.pk)
-        except models.EnterpriseCustomerUser.DoesNotExist:
-            raise serializers.ValidationError("User has no EnterpriseCustomerUser")
+        except models.EnterpriseCustomerUser.DoesNotExist as no_user_error:
+            error_message = '[Enterprise API] User has no EnterpriseCustomerUser. User: {}'.format(value)
+            LOGGER.error(error_message)
+            raise serializers.ValidationError("User has no EnterpriseCustomerUser") from no_user_error
 
         self.enterprise_customer_user = enterprise_customer_user
         return value
@@ -169,11 +240,30 @@ class EnterpriseCourseEnrollmentWriteSerializer(serializers.ModelSerializer):
         __, created = models.EnterpriseCourseEnrollment.objects.get_or_create(
             enterprise_customer_user=self.enterprise_customer_user,
             course_id=course_id,
+            defaults={
+                'source': models.EnterpriseEnrollmentSource.get_source(
+                    models.EnterpriseEnrollmentSource.OFFER_REDEMPTION
+                )
+            }
         )
         if created:
             track_enrollment('rest-api-enrollment', self.enterprise_customer_user.user_id, course_id)
 
 
+class LicensedEnterpriseCourseEnrollmentReadOnlySerializer(serializers.ModelSerializer):
+    """
+    Serializer for LicensedEnterpriseCourseEnrollment model.
+    """
+
+    enterprise_course_enrollment = EnterpriseCourseEnrollmentReadOnlySerializer(read_only=True)
+
+    class Meta:
+        model = models.LicensedEnterpriseCourseEnrollment
+        fields = (
+            'enterprise_course_enrollment', 'license_uuid'
+        )
+
+
 class EnterpriseCustomerCatalogSerializer(serializers.ModelSerializer):
     """
     Serializer for the ``EnterpriseCustomerCatalog`` model.
@@ -205,7 +295,7 @@ class EnterpriseCustomerCatalogDetailSerializer(EnterpriseCustomerCatalogSeriali
         request = self.context['request']
         enterprise_customer = instance.enterprise_customer
 
-        representation = super(EnterpriseCustomerCatalogDetailSerializer, self).to_representation(instance)
+        representation = super().to_representation(instance)
 
         # Retrieve the EnterpriseCustomerCatalog search results from the discovery service.
         paginated_content = instance.get_paginated_content(request.GET)
@@ -222,6 +312,8 @@ class EnterpriseCustomerCatalogDetailSerializer(EnterpriseCustomerCatalogSeriali
             # Add the Enterprise enrollment URL to each content item returned from the discovery service.
             if content_type == 'course':
                 item['enrollment_url'] = instance.get_course_enrollment_url(item['key'])
+                item['active'] = has_course_run_available_for_enrollment(item['course_runs'])
+                item['end_date'] = get_last_course_run_end_date(item['course_runs'])
             if content_type == 'courserun':
                 item['enrollment_url'] = instance.get_course_run_enrollment_url(item['key'])
             if content_type == 'program':
@@ -253,12 +345,13 @@ class EnterpriseCustomerUserReadOnlySerializer(serializers.ModelSerializer):
     class Meta:
         model = models.EnterpriseCustomerUser
         fields = (
-            'id', 'enterprise_customer', 'user_id', 'user', 'data_sharing_consent_records'
+            'id', 'enterprise_customer', 'active', 'user_id', 'user', 'data_sharing_consent_records', 'groups'
         )
 
     user = UserSerializer()
     enterprise_customer = EnterpriseCustomerSerializer()
     data_sharing_consent_records = serializers.SerializerMethodField()
+    groups = serializers.SerializerMethodField()
 
     def get_data_sharing_consent_records(self, obj):
         """
@@ -272,6 +365,14 @@ class EnterpriseCustomerUserReadOnlySerializer(serializers.ModelSerializer):
         """
         return [record.serialize() for record in obj.data_sharing_consent_records]
 
+    def get_groups(self, obj):
+        """
+        Return the enterprise related django groups that this user is a part of.
+        """
+        if obj.user:
+            return [group.name for group in obj.user.groups.filter(name__in=ENTERPRISE_PERMISSION_GROUPS)]
+        return []
+
 
 class EnterpriseCustomerUserWriteSerializer(serializers.ModelSerializer):
     """
@@ -281,10 +382,11 @@ class EnterpriseCustomerUserWriteSerializer(serializers.ModelSerializer):
     class Meta:
         model = models.EnterpriseCustomerUser
         fields = (
-            'enterprise_customer', 'username'
+            'enterprise_customer', 'username', 'active'
         )
 
     username = serializers.CharField(max_length=30)
+    active = serializers.BooleanField(required=False, default=True)
     user = None
 
     def validate_username(self, value):
@@ -293,8 +395,11 @@ class EnterpriseCustomerUserWriteSerializer(serializers.ModelSerializer):
         """
         try:
             self.user = User.objects.get(username=value)
-        except User.DoesNotExist:
-            raise serializers.ValidationError("User does not exist")
+        except User.DoesNotExist as no_user_error:
+            error_message = ('[Enterprise API] Saving to EnterpriseCustomerUser failed'
+                             ' due to non-existing user. User: {}').format(value)
+            LOGGER.error(error_message)
+            raise serializers.ValidationError("User does not exist") from no_user_error
 
         return value
 
@@ -303,53 +408,70 @@ class EnterpriseCustomerUserWriteSerializer(serializers.ModelSerializer):
         Save the EnterpriseCustomerUser.
         """
         enterprise_customer = self.validated_data['enterprise_customer']
+        active = self.validated_data.get('active', True)
+        defaults = {'active': active}
 
-        ecu = models.EnterpriseCustomerUser(
+        __, created = models.EnterpriseCustomerUser.objects.update_or_create(
             user_id=self.user.pk,
             enterprise_customer=enterprise_customer,
+            defaults=defaults,
         )
-        ecu.save()
+        if not created and active:
+            # if learner has activated enterprise we need to de-activate other enterprises learner is linked to
+            models.EnterpriseCustomerUser.inactivate_other_customers(self.user.pk, enterprise_customer)
 
 
-class EnterpriseCustomerUserEntitlementSerializer(ImmutableStateSerializer):
+class PendingEnterpriseCustomerUserSerializer(serializers.ModelSerializer):
     """
-    Serializer for the entitlements of EnterpriseCustomerUser.
-
-    This Serializer is for read only endpoint of enterprise learner's entitlements
-    It will ignore any state changing requests like POST, PUT and PATCH.
+    Serializer for writing to the PendingEnterpriseCustomerUser model.
     """
 
-    entitlements = serializers.ListField(
-        child=serializers.DictField()
-    )
+    class Meta:
+        model = models.PendingEnterpriseCustomerUser
+        fields = (
+            'enterprise_customer', 'user_email'
+        )
 
-    user = UserSerializer(read_only=True)
-    enterprise_customer = EnterpriseCustomerSerializer(read_only=True)
+    def validate(self, attrs):
+        """
+        Validate if the EnterpriseCustomerUser record already exists.
+        """
+        enterprise_customer = attrs.get('enterprise_customer')
+        user_email = attrs.get('user_email')
+        try:
+            user = User.objects.get(email=user_email)
+            models.EnterpriseCustomerUser.objects.get(
+                user_id=user.pk,
+                enterprise_customer=enterprise_customer
+            )
+        except (User.DoesNotExist, models.EnterpriseCustomerUser.DoesNotExist):
+            pass
+        else:
+            raise serializers.ValidationError('EnterpriseCustomerUser record already exists')
 
+        return attrs
 
-class CourseCatalogApiResponseReadOnlySerializer(ImmutableStateSerializer):
-    """
-    Serializer for enterprise customer catalog.
-    """
-
-    # pylint: disable=invalid-name
-    id = serializers.IntegerField(read_only=True, help_text=_('Enterprise course catalog primary key.'))
-    name = serializers.CharField(help_text=_('Catalog name'))
-    query = serializers.CharField(help_text=_('Query to retrieve catalog contents'))
-    courses_count = serializers.IntegerField(read_only=True, help_text=_('Number of courses contained in this catalog'))
-    viewers = serializers.ListField(
-        allow_null=True, allow_empty=True, required=False,
-        help_text=_('Usernames of users with explicit access to view this catalog'),
-        style={'base_template': 'input.html'},
-        child=serializers.CharField(),
-    )
-
-
-class EnterpriseCatalogCoursesReadOnlySerializer(ResponsePaginationSerializer, EnterpriseCourseContextSerializerMixin):
-    """
-    Serializer for enterprise customer catalog courses.
-    """
-    pass
+    def save(self):  # pylint: disable=arguments-differ
+        """
+        Save the PendingEnterpriseCustomerUser, or EnterpriseCustomerUser
+        if a user with the validated_email already exists.
+        """
+        enterprise_customer = self.validated_data['enterprise_customer']
+        user_email = self.validated_data['user_email']
+        try:
+            user = User.objects.get(email=user_email)
+            defaults = {'active': user.is_active}
+            __, created = models.EnterpriseCustomerUser.objects.update_or_create(
+                user_id=user.pk,
+                enterprise_customer=enterprise_customer,
+                defaults=defaults,
+            )
+        except User.DoesNotExist:
+            __, created = models.PendingEnterpriseCustomerUser.objects.update_or_create(
+                user_email=user_email,
+                enterprise_customer=enterprise_customer,
+            )
+        return created
 
 
 class CourseDetailSerializer(ImmutableStateSerializer):
@@ -444,20 +566,41 @@ class EnterpriseCustomerReportingConfigurationSerializer(serializers.ModelSerial
     """
     Serializer for EnterpriseCustomerReportingConfiguration model.
     """
-
     class Meta:
         model = models.EnterpriseCustomerReportingConfiguration
         fields = (
-            'enterprise_customer', 'active', 'delivery_method', 'email', 'frequency', 'day_of_month', 'day_of_week',
-            'hour_of_day', 'encrypted_password', 'sftp_hostname', 'sftp_port', 'sftp_username',
-            'encrypted_sftp_password', 'sftp_file_path', 'data_type', 'report_type',
+            'enterprise_customer', 'enterprise_customer_id', 'active', 'delivery_method', 'email', 'frequency',
+            'day_of_month', 'day_of_week', 'hour_of_day', 'include_date', 'encrypted_password', 'sftp_hostname',
+            'sftp_port', 'sftp_username', 'encrypted_sftp_password', 'sftp_file_path', 'data_type', 'report_type',
+            'pgp_encryption_key', 'enterprise_customer_catalogs', 'uuid'
         )
 
-    enterprise_customer = EnterpriseCustomerSerializer()
+    encrypted_password = serializers.CharField(required=False, allow_blank=False, read_only=False)
+    encrypted_sftp_password = serializers.CharField(required=False, allow_blank=False, read_only=False)
+    enterprise_customer = EnterpriseCustomerSerializer(read_only=True)
+    enterprise_customer_id = serializers.PrimaryKeyRelatedField(
+        queryset=models.EnterpriseCustomer.objects.all(),  # pylint: disable=no-member
+        source='enterprise_customer',
+        write_only=True
+    )
+    enterprise_customer_catalogs = EnterpriseCustomerCatalogSerializer(many=True, read_only=True)
     email = serializers.ListField(
+        default=[],
         child=serializers.EmailField()
     )
 
+    def validate(self, data):  # pylint: disable=arguments-differ
+        delivery_method = data.get('delivery_method')
+        if not delivery_method and self.instance:
+            delivery_method = self.instance.delivery_method
+
+        # in case of email delivery, email field should be populated with valid email(s)
+        if delivery_method == models.EnterpriseCustomerReportingConfiguration.DELIVERY_METHOD_EMAIL:
+            if 'email' in data and not bool(data['email']):
+                raise serializers.ValidationError({'email': ['This field is required']})
+
+        return data
+
 
 # pylint: disable=abstract-method
 class EnterpriseCustomerCourseEnrollmentsListSerializer(serializers.ListSerializer):
@@ -522,13 +665,14 @@ class EnterpriseCustomerCourseEnrollmentsSerializer(serializers.Serializer):
     This is mainly useful for implementing validation when performing enrollment operations.
     """
 
-    class Meta:  # pylint: disable=old-style-class
+    class Meta:
         list_serializer_class = EnterpriseCustomerCourseEnrollmentsListSerializer
 
     lms_user_id = serializers.CharField(required=False, write_only=True)
     tpa_user_id = serializers.CharField(required=False, write_only=True)
     user_email = serializers.EmailField(required=False, write_only=True)
     course_run_id = serializers.CharField(required=True, write_only=True)
+    cohort = serializers.CharField(required=False, write_only=True)
     course_mode = serializers.ChoiceField(
         choices=(
             ('audit', 'audit'),
@@ -540,6 +684,7 @@ class EnterpriseCustomerCourseEnrollmentsSerializer(serializers.Serializer):
     )
     email_students = serializers.BooleanField(default=False, required=False, write_only=True)
     detail = serializers.CharField(read_only=True)
+    is_active = serializers.BooleanField(required=False, default=True, write_only=True)
 
     def create(self, validated_data):
         """
@@ -551,25 +696,102 @@ class EnterpriseCustomerCourseEnrollmentsSerializer(serializers.Serializer):
         user_email = validated_data.get('user_email')
         course_run_id = validated_data.get('course_run_id')
         course_mode = validated_data.get('course_mode')
+        cohort = validated_data.get('cohort')
         email_students = validated_data.get('email_students')
+        is_active = validated_data.get('is_active')
+        discount = enterprise_customer.default_contract_discount or 0
+        LOGGER.info(
+            "[Enrollment-API] Received a call with the following parameters. lms_user: [{lms_user}], "
+            "tpa_user: [{tpa_user}], user_email: {user_email}, course_run_id: {course_run_id}, "
+            "course_mode: {course_mode}, is_active: {is_active}, discount: {discount}".format(
+                lms_user=lms_user.id if lms_user else None,
+                tpa_user=tpa_user.id if tpa_user else None,
+                user_email=user_email.id if isinstance(user_email, models.EnterpriseCustomerUser) else user_email,
+                course_run_id=course_run_id,
+                course_mode=course_mode,
+                is_active=is_active,
+                discount=discount
+            )
+        )
 
         enterprise_customer_user = lms_user or tpa_user or user_email
 
         if isinstance(enterprise_customer_user, models.EnterpriseCustomerUser):
             validated_data['enterprise_customer_user'] = enterprise_customer_user
             try:
-                enterprise_customer_user.enroll(course_run_id, course_mode)
-            except (utils.CourseEnrollmentDowngradeError, HttpClientError) as exc:
+                if is_active:
+                    LOGGER.info(
+                        "[Enrollment-API] Enrolling the enterprise learner [{learner_id}] in course [{course_run_id}] "
+                        "in [{course_mode}] mode".format(
+                            learner_id=enterprise_customer_user.id,
+                            course_run_id=course_run_id,
+                            course_mode=course_mode
+                        )
+                    )
+                    enterprise_customer_user.enroll(
+                        course_run_id,
+                        course_mode,
+                        cohort=cohort,
+                        source_slug=models.EnterpriseEnrollmentSource.API,
+                        discount_percentage=discount
+                    )
+                else:
+                    LOGGER.info(
+                        "[Enrollment-API] Un-enrolling the enterprise learner [{learner_id}] in course "
+                        "[{course_run_id}]".format(
+                            learner_id=enterprise_customer_user.id,
+                            course_run_id=course_run_id,
+                        )
+                    )
+                    enterprise_customer_user.unenroll(course_run_id)
+            except (CourseEnrollmentDowngradeError, CourseEnrollmentPermissionError, HttpClientError) as exc:
+                error_message = (
+                    '[Enterprise API] An exception occurred while enrolling the user.'
+                    ' EnterpriseCustomer: {enterprise_customer}, LmsUser: {lms_user}, TpaUser: {tpa_user},'
+                    ' UserEmail: {user_email}, CourseRun: {course_run_id}, CourseMode {course_mode}, Message: {exc}.'
+                ).format(
+                    enterprise_customer=enterprise_customer,
+                    lms_user=lms_user,
+                    tpa_user=tpa_user,
+                    user_email=user_email,
+                    course_run_id=course_run_id,
+                    course_mode=course_mode,
+                    exc=str(exc)
+                )
+                LOGGER.error(error_message)
                 validated_data['detail'] = str(exc)
                 return validated_data
 
-            track_enrollment('enterprise-customer-enrollment-api', enterprise_customer_user.user_id, course_run_id)
+            if is_active:
+                track_enrollment('enterprise-customer-enrollment-api', enterprise_customer_user.user_id, course_run_id)
         else:
-            enterprise_customer_user = enterprise_customer.enroll_user_pending_registration(
-                user_email,
-                course_mode,
-                course_run_id
-            )
+            if is_active:
+                LOGGER.info(
+                    "[Enrollment-API] Creating the pending enrollment for [{email}] in course [{course_run_id}] "
+                    "in [{course_mode}] mode".format(
+                        email=user_email,
+                        course_run_id=course_run_id,
+                        course_mode=course_mode
+                    )
+                )
+                enterprise_customer_user = enterprise_customer.enroll_user_pending_registration(
+                    user_email,
+                    course_mode,
+                    course_run_id,
+                    cohort=cohort,
+                    enrollment_source=models.EnterpriseEnrollmentSource.get_source(
+                        models.EnterpriseEnrollmentSource.API
+                    ),
+                    discount=discount
+                )
+            else:
+                LOGGER.info(
+                    "[Enrollment-API] Removing the pending enrollment for [{email}] in course [{course_run_id}]".format(
+                        email=user_email,
+                        course_run_id=course_run_id
+                    )
+                )
+                enterprise_customer.clear_pending_registration(user_email, course_run_id)
 
         if email_students:
             enterprise_customer.notify_enrolled_learners(
@@ -579,6 +801,18 @@ class EnterpriseCustomerCourseEnrollmentsSerializer(serializers.Serializer):
             )
 
         validated_data['detail'] = 'success'
+        LOGGER.info(
+            "[Enrollment-API] Returning success for a call with the following parameters. lms_user: [{lms_user}], "
+            "tpa_user: [{tpa_user}], user_email: {user_email}, course_run_id: {course_run_id}, "
+            "course_mode: {course_mode}, is_active: {is_active}".format(
+                lms_user=lms_user.id if lms_user else None,
+                tpa_user=tpa_user.id if tpa_user else None,
+                user_email=user_email.id if isinstance(user_email, models.EnterpriseCustomerUser) else user_email,
+                course_run_id=course_run_id,
+                course_mode=course_mode,
+                is_active=is_active
+            )
+        )
 
         return validated_data
 
@@ -608,7 +842,7 @@ class EnterpriseCustomerCourseEnrollmentsSerializer(serializers.Serializer):
         enterprise_customer = self.context.get('enterprise_customer')
 
         try:
-            tpa_client = ThirdPartyAuthApiClient()
+            tpa_client = ThirdPartyAuthApiClient(self.context['request_user'])
             username = tpa_client.get_username_from_remote_id(
                 enterprise_customer.identity_provider, value
             )
@@ -648,6 +882,13 @@ class EnterpriseCustomerCourseEnrollmentsSerializer(serializers.Serializer):
         enterprise_customer = self.context.get('enterprise_customer')
 
         if not enterprise_customer.catalog_contains_course(value):
+            error_message = ('[Enterprise API] The course run id is not in the catalog for the Enterprise Customer.'
+                             ' EnterpriseCustomer: {enterprise_uuid}, EnterpriseName: {enterprise_name},'
+                             ' CourseRun: {course_run_id}').format(
+                                 course_run_id=value,
+                                 enterprise_uuid=enterprise_customer.uuid,
+                                 enterprise_name=enterprise_customer.name)
+            LOGGER.warning(error_message)
             raise serializers.ValidationError(
                 'The course run id {course_run_id} is not in the catalog '
                 'for Enterprise Customer {enterprise_customer}'.format(
@@ -666,9 +907,48 @@ class EnterpriseCustomerCourseEnrollmentsSerializer(serializers.Serializer):
         tpa_user_id = data.get('tpa_user_id')
         user_email = data.get('user_email')
         if not lms_user_id and not tpa_user_id and not user_email:
+            error_message = ('[Enterprise API] ID missing for mapping to an EnterpriseCustomerUser.'
+                             ' LmsUser: {lms_user_id}, TpaUser: {tpa_user_id}, UserEmail: {user_email}').format(
+                                 lms_user_id=lms_user_id,
+                                 tpa_user_id=tpa_user_id,
+                                 user_email=user_email)
+            LOGGER.error(error_message)
             raise serializers.ValidationError(
                 'At least one of the following fields must be specified and map to an EnterpriseCustomerUser: '
                 'lms_user_id, tpa_user_id, user_email'
             )
 
         return data
+
+
+# pylint: disable=abstract-method
+class EnterpriseCustomerBulkEnrollmentsSerializer(serializers.Serializer):
+    """
+    Serializes a email_csv or email field for bulk enrollment requests.
+    """
+    email = serializers.CharField(required=False)
+    email_csv = Base64EmailCSVField(required=False)
+    course_run_key = serializers.CharField(required=False)
+    course_mode = serializers.ChoiceField(
+        choices=[
+            ("audit", _("Audit")),
+            ("verified", _("Verified")),
+            ("professional", _("Professional Education")),
+            ("no-id-professional", _("Professional Education (no ID)")),
+            ("credit", _("Credit")),
+            ("honor", _("Honor")),
+        ],
+        required=False,
+    )
+    reason = serializers.CharField(required=False)
+    salesforce_id = serializers.CharField(required=False)
+    discount = serializers.DecimalField(None, 5, required=False)
+    notify = serializers.BooleanField(default=False)
+
+    def create(self, validated_data):
+        return validated_data
+
+    def validate(self, data):  # pylint: disable=arguments-differ
+        if not data.get('email') and not data.get('email_csv'):
+            raise serializers.ValidationError('Must include either email or email_csv in request.')
+        return data
