@@ -2,7 +2,6 @@
 """
 Utilities to get details from the course catalog API.
 """
-from __future__ import absolute_import, unicode_literals
 
 from logging import getLogger
 
@@ -13,18 +12,20 @@ from opaque_keys.edx.keys import CourseKey
 from requests.exceptions import ConnectionError, Timeout  # pylint: disable=redefined-builtin
 
 from django.conf import settings
+from django.core.cache import cache
 from django.core.exceptions import ImproperlyConfigured, ObjectDoesNotExist
 from django.utils.translation import ugettext_lazy as _
 
-from enterprise.utils import (
-    MultipleProgramMatchError,
-    NotConnectedToOpenEdX,
-    get_configuration_value_for_site,
-    parse_course_key,
-)
+from enterprise import utils
+from enterprise.utils import NotConnectedToOpenEdX, get_configuration_value_for_site
 
 try:
-    from openedx.core.lib.token_utils import JwtBuilder
+    import cPickle as pickle
+except ImportError:
+    import pickle
+
+try:
+    from openedx.core.djangoapps.oauth_dispatch import jwt as JwtBuilder
 except ImportError:
     JwtBuilder = None
 
@@ -52,19 +53,16 @@ def course_discovery_api_client(user, catalog_url):
               "installed in an Open edX environment.")
         )
 
-    scopes = ['email', 'profile']
-    expires_in = settings.OAUTH_ID_TOKEN_EXPIRATION
-    jwt = JwtBuilder(user).build_token(scopes, expires_in)
+    jwt = JwtBuilder.create_jwt_for_user(user)
     return EdxRestApiClient(catalog_url, jwt=jwt)
 
 
-class CourseCatalogApiClient(object):
+class CourseCatalogApiClient:
     """
     Object builds an API client to make calls to the Catalog API.
     """
 
     SEARCH_ALL_ENDPOINT = 'search/all/'
-    CATALOGS_ENDPOINT = 'catalogs'
     CATALOGS_COURSES_ENDPOINT = 'catalogs/{}/courses/'
     COURSES_ENDPOINT = 'courses'
     COURSE_RUNS_ENDPOINT = 'course_runs'
@@ -93,108 +91,166 @@ class CourseCatalogApiClient(object):
             )
 
         self.user = user
-        catalog_url = get_configuration_value_for_site(
+        self.catalog_url = get_configuration_value_for_site(
             site,
             'COURSE_CATALOG_API_URL',
             settings.COURSE_CATALOG_API_URL
         )
-        self.client = course_discovery_api_client(user, catalog_url)
+        self.client = course_discovery_api_client(user, self.catalog_url)
 
-    def get_search_results(self, querystring=None, traverse_pagination=True):
+    @staticmethod
+    def traverse_pagination(response, endpoint, content_filter_query, query_params):
         """
-        Return results from the discovery service's search/all endpoint.
+        Traverse a paginated API response and extracts and concatenates "results" returned by API.
 
         Arguments:
-            querystring (dict): Querystring parameters used to filter search results.
+            response (dict): API response object.
+            endpoint (Slumber.Resource): API endpoint object.
+            content_filter_query (dict): query parameters used to filter catalog results.
+            query_params (dict): query parameters used to paginate results.
+
+        Returns:
+            list: all the results returned by the API.
+        """
+        results = response.get('results', [])
+
+        page = 1
+        while response.get('next'):
+            page += 1
+            response = endpoint().post(content_filter_query, **dict(query_params, page=page))
+            results += response.get('results', [])
+
+        return results
+
+    def get_catalog_results_from_discovery(self, content_filter_query, query_params=None, traverse_pagination=False):
+        """
+            Return results from the discovery service's search/all endpoint."""
+
+        endpoint = getattr(self.client, self.SEARCH_ALL_ENDPOINT)
+        response = endpoint().post(data=content_filter_query, **query_params)
+        if traverse_pagination:
+            response['results'] = self.traverse_pagination(response, endpoint, content_filter_query, query_params)
+            response['next'] = response['previous'] = None
+        return response
+
+    def get_catalog_results(self, content_filter_query, query_params=None, traverse_pagination=False):
+        """
+            Return results from the cache or discovery service's search/all endpoint.
+        Arguments:
+            content_filter_query (dict): query parameters used to filter catalog results.
+            query_params (dict): query parameters used to paginate results.
             traverse_pagination (bool): True to return all results, False to return the paginated response.
-                                        Defaults to True.
-        Returns:
-            list or dict: The paginated response dict if traverse_pagination is False, otherwise the extracted
-                          results list.
-
-        """
-        return self._load_data(
-            self.SEARCH_ALL_ENDPOINT,
-            default=[],
-            querystring=querystring,
-            traverse_pagination=traverse_pagination,
-            many=False,
-        )
-
-    def get_all_catalogs(self):
-        """
-        Return a list of all course catalogs, including name and ID.
+                                        Defaults to False.
 
         Returns:
-            list: List of catalogs available for the user.
-
+            dict: Paginated response or all the records.
         """
-        return self._load_data(
-            self.CATALOGS_ENDPOINT,
-            default=[]
-        )
+        query_params = query_params or {}
 
-    def get_catalog(self, catalog_id):
+        try:
+            cache_key = utils.get_cache_key(
+                service='discovery',
+                endpoint=self.SEARCH_ALL_ENDPOINT,
+                query=content_filter_query,
+                traverse_pagination=traverse_pagination,
+                **query_params
+            )
+            response = cache.get(cache_key)
+            if not response:
+                LOGGER.info(
+                    'ENT-2390-1 | Calling discovery service for search/all/ '
+                    'data with content_filter_query %s and query_params %s',
+                    content_filter_query,
+                    query_params,
+                )
+                # Response is not cached, so make a call.
+                response = self.get_catalog_results_from_discovery(
+                    content_filter_query,
+                    query_params,
+                    traverse_pagination
+                )
+                response_as_string = pickle.dumps(response)
+                LOGGER.info(
+                    'ENT-2489 | Response from content_filter_query %s is %d bytes long.',
+                    content_filter_query,
+                    len(response_as_string)
+                )
+                cache.set(cache_key, response, settings.ENTERPRISE_API_CACHE_TIMEOUT)
+            else:
+                LOGGER.info(
+                    'ENT-2390-2 | Got search/all/ data from the cache with '
+                    'content_filter_query %s and query_params %s',
+                    content_filter_query,
+                    query_params,
+                )
+        except Exception as ex:  # pylint: disable=broad-except
+            LOGGER.exception(
+                'Attempted to call course-discovery search/all/ endpoint with the following parameters: '
+                'content_filter_query: %s, query_params: %s, traverse_pagination: %s. '
+                'Failed to retrieve data from the catalog API. content -- [%s]',
+                content_filter_query,
+                query_params,
+                traverse_pagination,
+                getattr(ex, 'content', '')
+            )
+            # We need to bubble up failures when we encounter them instead of masking them!
+            raise ex
+
+        return response
+
+    def get_course_id(self, course_identifier):
         """
-        Return specified course catalog.
+        Return the course id for the given course identifier.  The `course_identifier` may be a course id or a course
+        run id; in either case the course id will be returned.
+
+        The 'course id' is the identifier for a course (ex. edX+DemoX)
+        The 'course run id' is the identifier for a run of a course (ex. edX+DemoX+demo_run)
+
+        Arguments:
+            course_identifier (str): The course id or course run id
 
         Returns:
-            dict: catalog details if it is available for the user.
-
+            (str): course id
         """
-        return self._load_data(
-            self.CATALOGS_ENDPOINT,
-            default=[],
-            resource_id=catalog_id
+
+        try:
+            CourseKey.from_string(course_identifier)
+        except InvalidKeyError:
+            # An `InvalidKeyError` is thrown if `course_identifier` is not in the proper format for a course run id.
+            # Since `course_identifier` is not a course run id we assume `course_identifier` is the  course id.
+            return course_identifier
+
+        # If here, `course_identifier` must be a course run id.
+        # We cannot use `CourseKey.from_string` to find the course id because that method assumes the course id is
+        # always a substring of the course run id and this is not always the case.  The only reliable way to determine
+        # which courses are associated with a given course run id is by by calling the discovery service.
+        course_run_data = self.get_course_run(course_identifier)
+        if 'course' in course_run_data:
+            return course_run_data['course']
+
+        LOGGER.info(
+            "Could not find course_key for course identifier [%s].", course_identifier
         )
+        return None
 
-    def get_paginated_catalog_courses(self, catalog_id, querystring=None):
+    def get_course_run_identifiers(self, course_run_id):
         """
-        Return paginated response for all catalog courses.
-
-        Returns:
-            dict: API response with links to next and previous pages.
-
+        Return all course and course run keys and uuids for the specified course run id
         """
-        return self._load_data(
-            self.CATALOGS_COURSES_ENDPOINT.format(catalog_id),
-            default=[],
-            querystring=querystring,
-            traverse_pagination=False,
-            many=False,
-        )
-
-    def get_paginated_catalogs(self, querystring=None):
-        """
-        Return a paginated list of course catalogs, including name and ID.
-
-        Returns:
-            dict: Paginated response containing catalogs available for the user.
-
-        """
-        return self._load_data(
-            self.CATALOGS_ENDPOINT,
-            default=[],
-            querystring=querystring,
-            traverse_pagination=False,
-            many=False
-        )
-
-    def get_catalog_courses(self, catalog_id):
-        """
-        Return the courses included in a single course catalog by ID.
-
-        Args:
-            catalog_id (int): The catalog ID we want to retrieve.
-
-        Returns:
-            list: Courses of the catalog in question
-
-        """
-        return self._load_data(
-            self.CATALOGS_COURSES_ENDPOINT.format(catalog_id),
-            default=[]
-        )
+        course, course_run = self.get_course_and_course_run(course_run_id)
+        identifiers = {
+            'course_key': None,
+            'course_uuid': None,
+            'course_run_key': None,
+            'course_run_uuid': None,
+        }
+        if course:
+            identifiers['course_key'] = course.get('key')
+            identifiers['course_uuid'] = course.get('uuid')
+        if course_run:
+            identifiers['course_run_key'] = course_run.get('key')
+            identifiers['course_run_uuid'] = course_run.get('uuid')
+        return identifiers
 
     def get_course_and_course_run(self, course_run_id):
         """
@@ -206,18 +262,20 @@ class CourseCatalogApiClient(object):
         Returns:
             tuple: The course metadata and the course run metadata.
         """
-        # Parse the course ID from the course run ID.
-        course_id = parse_course_key(course_run_id)
+        course_id = self.get_course_id(course_run_id)
         # Retrieve the course metadata from the catalog service.
         course = self.get_course_details(course_id)
 
+        # Return the specified course run from the returned course detail container
         course_run = None
         if course:
-            # Find the specified course run.
-            course_run = None
-            course_runs = [course_run for course_run in course['course_runs'] if course_run['key'] == course_run_id]
+            course_runs = course.get('course_runs')
+            matching_course_runs = None
             if course_runs:
-                course_run = course_runs[0]
+                matching_course_runs = [course_run for course_run in course_runs
+                                        if course_run.get('key') == course_run_id]
+            if matching_course_runs:
+                course_run = matching_course_runs[0]
 
         return course, course_run
 
@@ -251,29 +309,10 @@ class CourseCatalogApiClient(object):
         """
         return self._load_data(
             self.COURSE_RUNS_ENDPOINT,
-            resource_id=course_run_id
+            resource_id=course_run_id,
+            long_term_cache=True
         )
 
-    def get_program_by_title(self, program_title):
-        """
-        Return single program by name, or None if not found.
-
-        Arguments:
-            program_title(string): Program title as seen by students and in Course Catalog Admin
-
-        Returns:
-            dict: Program data provided by Course Catalog API
-
-        """
-        all_programs = self._load_data(self.PROGRAMS_ENDPOINT, default=[])
-        matching_programs = [program for program in all_programs if program.get('title') == program_title]
-        if len(matching_programs) > 1:
-            raise MultipleProgramMatchError(len(matching_programs))
-        elif len(matching_programs) == 1:
-            return matching_programs[0]
-        else:
-            return None
-
     def get_program_by_uuid(self, program_uuid):
         """
         Return single program by UUID, or None if not found.
@@ -324,86 +363,6 @@ class CourseCatalogApiClient(object):
             default=None,
         )
 
-    def get_common_course_modes(self, course_run_ids):
-        """
-        Find common course modes for a set of course runs.
-
-        This function essentially returns an intersection of types of seats available
-        for each course run.
-
-        Arguments:
-            course_run_ids(Iterable[str]): Target Course run IDs.
-
-        Returns:
-            set: course modes found in all given course runs
-
-        Examples:
-            # run1 has prof and audit, run 2 has the same
-            get_common_course_modes(['course-v1:run1', 'course-v1:run2'])
-            {'prof', 'audit'}
-
-            # run1 has prof and audit, run 2 has only prof
-            get_common_course_modes(['course-v1:run1', 'course-v1:run2'])
-            {'prof'}
-
-            # run1 has prof and audit, run 2 honor
-            get_common_course_modes(['course-v1:run1', 'course-v1:run2'])
-            {}
-
-            # run1 has nothing, run2 has prof
-            get_common_course_modes(['course-v1:run1', 'course-v1:run2'])
-            {}
-
-            # run1 has prof and audit, run 2 prof, run3 has audit
-            get_common_course_modes(['course-v1:run1', 'course-v1:run2', 'course-v1:run3'])
-            {}
-
-            # run1 has nothing, run 2 prof, run3 has prof
-            get_common_course_modes(['course-v1:run1', 'course-v1:run2', 'course-v1:run3'])
-            {}
-
-        """
-        available_course_modes = None
-        for course_run_id in course_run_ids:
-            course_run = self.get_course_run(course_run_id) or {}
-            course_run_modes = {seat.get('type') for seat in course_run.get('seats', [])}
-
-            if available_course_modes is None:
-                available_course_modes = course_run_modes
-            else:
-                available_course_modes &= course_run_modes
-
-            if not available_course_modes:
-                return available_course_modes
-
-        return available_course_modes
-
-    def is_course_in_catalog(self, catalog_id, course_id):
-        """
-        Determine if the given course or course run ID is contained in the catalog with the given ID.
-
-        Args:
-            catalog_id (int): The ID of the catalog
-            course_id (str): The ID of the course or course run
-
-        Returns:
-            bool: Whether the course or course run is contained in the given catalog
-        """
-        try:
-            # Determine if we have a course run ID, rather than a plain course ID
-            course_run_id = str(CourseKey.from_string(course_id))
-        except InvalidKeyError:
-            course_run_id = None
-
-        endpoint = self.client.catalogs(catalog_id).contains
-
-        if course_run_id:
-            resp = endpoint.get(course_run_id=course_run_id)
-        else:
-            resp = endpoint.get(course_id=course_id)
-
-        return resp.get('courses', {}).get(course_id, False)
-
     def _load_data(self, resource, default=DEFAULT_VALUE_SAFEGUARD, **kwargs):
         """
         Load data from API client.
@@ -432,6 +391,19 @@ class CourseCatalogApiClient(object):
             return default_val
 
 
+def get_course_catalog_api_service_client(site=None):
+    """
+    Returns an instance of the CourseCatalogApiServiceClient
+
+    Args:
+        site: (Site)
+
+    Returns:
+        (CourseCatalogServiceClient)
+    """
+    return CourseCatalogApiServiceClient(site=site)
+
+
 class CourseCatalogApiServiceClient(CourseCatalogApiClient):
     """
     Catalog API client which uses the configured Catalog service user.
@@ -452,9 +424,11 @@ class CourseCatalogApiServiceClient(CourseCatalogApiClient):
         if catalog_integration.enabled:
             try:
                 user = catalog_integration.get_service_user()
-                super(CourseCatalogApiServiceClient, self).__init__(user, site)
-            except ObjectDoesNotExist:
-                raise ImproperlyConfigured(_("The configured CatalogIntegration service user does not exist."))
+                super().__init__(user, site)
+            except ObjectDoesNotExist as error:
+                raise ImproperlyConfigured(
+                    _("The configured CatalogIntegration service user does not exist.")
+                ) from error
         else:
             raise ImproperlyConfigured(_("There is no active CatalogIntegration."))
 
@@ -467,3 +441,27 @@ class CourseCatalogApiServiceClient(CourseCatalogApiClient):
             return bool(cls().get_program_by_uuid(program_uuid))
         except ImproperlyConfigured:
             return False
+
+
+class NoAuthDiscoveryClient:
+    """
+    Class to build a course discovery client to make calls to the discovery service.
+    """
+
+    API_BASE_URL = settings.COURSE_CATALOG_URL_ROOT
+    APPEND_SLASH = False
+
+    def __init__(self):
+        """
+        Create a course discovery client.
+        """
+        self.client = EdxRestApiClient(self.API_BASE_URL, append_slash=self.APPEND_SLASH)
+
+    def get_health(self):
+        """
+        Retrieve health details for course discovery service.
+
+        Returns:
+            dict: Response containing course discovery service health.
+        """
+        return self.client.health.get()
