@@ -3,8 +3,6 @@
 Client for communicating with the Enterprise API.
 """
 
-from __future__ import absolute_import, unicode_literals
-
 from collections import OrderedDict
 from logging import getLogger
 
@@ -30,32 +28,20 @@ class EnterpriseApiClient(JwtLmsApiClient):
 
     DEFAULT_VALUE_SAFEGUARD = object()
 
-    def get_content_metadata(self, enterprise_customer):
+    def get_content_metadata(self, enterprise_customer, enterprise_catalogs=None):
         """
         Return all content metadata contained in the catalogs associated with the EnterpriseCustomer.
 
         Arguments:
             enterprise_customer (EnterpriseCustomer): The EnterpriseCustomer to return content metadata for.
+            enterprise_catalogs (EnterpriseCustomerCatalog): Optional list of EnterpriseCustomerCatalog objects.
 
         Returns:
             list: List of dicts containing content metadata.
         """
         content_metadata = OrderedDict()
-
-        # TODO: This if block can be removed when we get rid of discovery service-based catalogs.
-        if enterprise_customer.catalog:
-            response = self._load_data(
-                self.ENTERPRISE_CUSTOMER_ENDPOINT,
-                detail_resource='courses',
-                resource_id=str(enterprise_customer.uuid),
-                traverse_pagination=True,
-            )
-            for course in response['results']:
-                for course_run in course['course_runs']:
-                    course_run['content_type'] = 'courserun'  # Make this look like a search endpoint result.
-                    content_metadata[course_run['key']] = course_run
-
-        for enterprise_customer_catalog in enterprise_customer.enterprise_customer_catalogs.all():
+        enterprise_customer_catalogs = enterprise_catalogs or enterprise_customer.enterprise_customer_catalogs.all()
+        for enterprise_customer_catalog in enterprise_customer_catalogs:
             response = self._load_data(
                 self.ENTERPRISE_CUSTOMER_CATALOGS_ENDPOINT,
                 resource_id=str(enterprise_customer_catalog.uuid),
@@ -67,7 +53,7 @@ class EnterpriseApiClient(JwtLmsApiClient):
                 content_id = utils.get_content_metadata_item_id(item)
                 content_metadata[content_id] = item
 
-        return content_metadata.values()
+        return list(content_metadata.values())
 
     @JwtLmsApiClient.refresh_token
     def _load_data(
