@@ -2,7 +2,6 @@
 """
 Utilities to get details from the course catalog API.
 """
-from __future__ import absolute_import, unicode_literals
 
 import datetime
 import logging
@@ -11,7 +10,6 @@ from time import time
 
 from edx_rest_api_client.client import EdxRestApiClient
 from opaque_keys.edx.keys import CourseKey
-from requests import Session
 from requests.exceptions import ConnectionError, Timeout  # pylint: disable=redefined-builtin
 from slumber.exceptions import HttpNotFoundError, SlumberBaseException
 
@@ -19,12 +17,7 @@ from django.conf import settings
 from django.utils import timezone
 
 from enterprise.constants import COURSE_MODE_SORT_ORDER, EXCLUDED_COURSE_MODES
-from enterprise.utils import NotConnectedToOpenEdX
-
-try:
-    from student.models import CourseEnrollment
-except ImportError:
-    CourseEnrollment = None
+from enterprise.utils import NotConnectedToOpenEdX, get_enterprise_worker_user
 
 try:
     from openedx.core.djangoapps.embargo import api as embargo_api
@@ -32,7 +25,7 @@ except ImportError:
     embargo_api = None
 
 try:
-    from openedx.core.lib.token_utils import JwtBuilder
+    from openedx.core.djangoapps.oauth_dispatch import jwt as JwtBuilder
 except ImportError:
     JwtBuilder = None
 
@@ -42,11 +35,11 @@ LMS_API_DATETIME_FORMAT = '%Y-%m-%dT%H:%M:%SZ'
 LMS_API_DATETIME_FORMAT_WITHOUT_TIMEZONE = '%Y-%m-%dT%H:%M:%S'
 
 
-class LmsApiClient(object):
+class NoAuthenticationLmsApiClient:
     """
     Object builds an API client to make calls to the edxapp LMS API.
 
-    Authenticates using settings.EDX_API_KEY.
+    Authentication is not required.
     """
 
     API_BASE_URL = settings.LMS_INTERNAL_ROOT_URL + '/api/'
@@ -54,16 +47,12 @@ class LmsApiClient(object):
 
     def __init__(self):
         """
-        Create an LMS API client, authenticated with the API token from Django settings.
+        Create an LMS API client.
         """
-        session = Session()
-        session.headers = {"X-Edx-Api-Key": settings.EDX_API_KEY}
-        self.client = EdxRestApiClient(
-            self.API_BASE_URL, append_slash=self.APPEND_SLASH, session=session
-        )
+        self.client = EdxRestApiClient(self.API_BASE_URL, append_slash=self.APPEND_SLASH)
 
 
-class JwtLmsApiClient(object):
+class JwtLmsApiClient:
     """
     LMS client authenticates using a JSON Web Token (JWT) for the given user.
     """
@@ -88,8 +77,7 @@ class JwtLmsApiClient(object):
             raise NotConnectedToOpenEdX("This package must be installed in an OpenEdX environment.")
 
         now = int(time())
-        scopes = ['profile', 'email']
-        jwt = JwtBuilder(self.user).build_token(scopes, self.expires_in)
+        jwt = JwtBuilder.create_jwt_for_user(self.user)
         self.client = EdxRestApiClient(
             self.API_BASE_URL, append_slash=self.APPEND_SLASH, jwt=jwt,
         )
@@ -117,7 +105,7 @@ class JwtLmsApiClient(object):
         return inner
 
 
-class EmbargoApiClient(object):
+class EmbargoApiClient:
     """
     Client interface for using the edx-platform embargo API.
     """
@@ -136,15 +124,21 @@ class EmbargoApiClient(object):
             )
             if redirect_url:
                 return redirect_url
+        return None
 
 
-class EnrollmentApiClient(LmsApiClient):
+class EnrollmentApiClient(JwtLmsApiClient):
     """
     Object builds an API client to make calls to the Enrollment API.
     """
 
     API_BASE_URL = settings.ENTERPRISE_ENROLLMENT_API_URL
 
+    def __init__(self, user=None):
+        user = user if user else get_enterprise_worker_user()
+        super().__init__(user)
+
+    @JwtLmsApiClient.refresh_token
     def get_course_details(self, course_id):
         """
         Query the Enrollment API for the course details of the given course_id.
@@ -186,6 +180,7 @@ class EnrollmentApiClient(LmsApiClient):
         # Sort slug weights in descending order
         return sorted(modes, key=slug_weight, reverse=True)
 
+    @JwtLmsApiClient.refresh_token
     def get_course_modes(self, course_id):
         """
         Query the Enrollment API for the specific course modes that are available for the given course_id.
@@ -201,6 +196,7 @@ class EnrollmentApiClient(LmsApiClient):
         modes = details.get('course_modes', [])
         return self._sort_course_modes([mode for mode in modes if mode['slug'] not in EXCLUDED_COURSE_MODES])
 
+    @JwtLmsApiClient.refresh_token
     def has_course_mode(self, course_run_id, mode):
         """
         Query the Enrollment API to see whether a course run has a given course mode available.
@@ -215,7 +211,8 @@ class EnrollmentApiClient(LmsApiClient):
         course_modes = self.get_course_modes(course_run_id)
         return any(course_mode for course_mode in course_modes if course_mode['slug'] == mode)
 
-    def enroll_user_in_course(self, username, course_id, mode):
+    @JwtLmsApiClient.refresh_token
+    def enroll_user_in_course(self, username, course_id, mode, cohort=None):
         """
         Call the enrollment API to enroll the user in the course specified by course_id.
 
@@ -223,6 +220,7 @@ class EnrollmentApiClient(LmsApiClient):
             username (str): The username by which the user goes on the OpenEdX platform
             course_id (str): The string value of the course's unique identifier
             mode (str): The enrollment mode which should be used for the enrollment
+            cohort (str): Add the user to this named cohort
 
         Returns:
             dict: A dictionary containing details of the enrollment, including course details, mode, username, etc.
@@ -232,10 +230,54 @@ class EnrollmentApiClient(LmsApiClient):
             {
                 'user': username,
                 'course_details': {'course_id': course_id},
+                'is_active': True,
                 'mode': mode,
+                'cohort': cohort,
             }
         )
 
+    @JwtLmsApiClient.refresh_token
+    def unenroll_user_from_course(self, username, course_id):
+        """
+        Call the enrollment API to unenroll the user in the course specified by course_id.
+        Args:
+            username (str): The username by which the user goes on the OpenEdx platform
+            course_id (str): The string value of the course's unique identifier
+        Returns:
+            bool: Whether the unenrollment succeeded
+        """
+        enrollment = self.get_course_enrollment(username, course_id)
+        if enrollment and enrollment['is_active']:
+            response = self.client.enrollment.post({
+                'user': username,
+                'course_details': {'course_id': course_id},
+                'is_active': False,
+                'mode': enrollment['mode']
+            })
+            return not response['is_active']
+
+        return False
+
+    @JwtLmsApiClient.refresh_token
+    def update_course_enrollment_mode_for_user(self, username, course_id, mode):
+        """
+        Call the enrollment API to update a user's course enrollment to the specified mode, e.g. "audit".
+
+        Args:
+            username (str): The username by which the user goes on the OpenEdx platform
+            course_id (str): The string value of the course's unique identifier
+            mode (str): The string value of the course mode, e.g. "audit"
+
+        Returns:
+            dict: A dictionary containing details of the enrollment, including course details, mode, username, etc.
+        """
+        return self.client.enrollment.post({
+            'user': username,
+            'course_details': {'course_id': course_id},
+            'mode': mode,
+        })
+
+    @JwtLmsApiClient.refresh_token
     def get_course_enrollment(self, username, course_id):
         """
         Query the enrollment API to get information about a single course enrollment.
@@ -270,6 +312,7 @@ class EnrollmentApiClient(LmsApiClient):
 
         return result
 
+    @JwtLmsApiClient.refresh_token
     def is_enrolled(self, username, course_run_id):
         """
         Query the enrollment API and determine if a learner is enrolled in a course run.
@@ -285,6 +328,7 @@ class EnrollmentApiClient(LmsApiClient):
         enrollment = self.get_course_enrollment(username, course_run_id)
         return enrollment is not None and enrollment.get('is_active', False)
 
+    @JwtLmsApiClient.refresh_token
     def get_enrolled_courses(self, username):
         """
         Query the enrollment API to get a list of the courses a user is enrolled in.
@@ -299,7 +343,7 @@ class EnrollmentApiClient(LmsApiClient):
         return self.client.enrollment.get(user=username)
 
 
-class CourseApiClient(LmsApiClient):
+class CourseApiClient(NoAuthenticationLmsApiClient):
     """
     Object builds an API client to make calls to the Course API.
     """
@@ -324,13 +368,14 @@ class CourseApiClient(LmsApiClient):
             return None
 
 
-class ThirdPartyAuthApiClient(LmsApiClient):
+class ThirdPartyAuthApiClient(JwtLmsApiClient):
     """
     Object builds an API client to make calls to the Third Party Auth API.
     """
 
     API_BASE_URL = settings.LMS_INTERNAL_ROOT_URL + '/api/third_party_auth/v0/'
 
+    @JwtLmsApiClient.refresh_token
     def get_remote_id(self, identity_provider, username):
         """
         Retrieve the remote identifier for the given username.
@@ -344,6 +389,7 @@ class ThirdPartyAuthApiClient(LmsApiClient):
         """
         return self._get_results(identity_provider, 'username', username, 'remote_id')
 
+    @JwtLmsApiClient.refresh_token
     def get_username_from_remote_id(self, identity_provider, remote_id):
         """
         Retrieve the remote identifier for the given username.
@@ -388,7 +434,7 @@ class GradesApiClient(JwtLmsApiClient):
     Note that this API client requires a JWT token, and so it keeps its token alive.
     """
 
-    API_BASE_URL = settings.LMS_INTERNAL_ROOT_URL + '/api/grades/v0/'
+    API_BASE_URL = settings.LMS_INTERNAL_ROOT_URL + '/api/grades/v1/'
     APPEND_SLASH = True
 
     @JwtLmsApiClient.refresh_token
@@ -415,13 +461,49 @@ class GradesApiClient(JwtLmsApiClient):
         * ``letter_grade``: A letter grade as defined in grading_policy (e.g. 'A' 'B' 'C' for 6.002x) or None
 
         """
-        results = self.client.course_grade(course_id).users().get(username=username)
+        results = self.client.courses(course_id).get(username=username)
         for row in results:
             if row.get('username') == username:
                 return row
 
         raise HttpNotFoundError('No grade record found for course={}, username={}'.format(course_id, username))
 
+    @JwtLmsApiClient.refresh_token
+    def get_course_assessment_grades(self, course_id, username):
+        """
+        Retrieve the assessment grades for the given username for the given course_id.
+
+        Args:
+        * ``course_id`` (str): The string value of the course's unique identifier
+        * ``username`` (str): The username ID identifying the user for which to retrieve the grade.
+
+        Raises:
+
+        HttpNotFoundError if no grade found for the given user+course.
+
+        Returns:
+
+        a list of dicts containing:
+
+        * ``attempted``: A boolean representing whether the learner has attempted the subsection yet.
+        * ``subsection_name``: String representation of the subsection's name.
+        * ``category``: String representation of the subsection's category.
+        * ``label``: String representation of the subsection's label.
+        * ``score_possible``: The total amount of points that the learner could have earned on the subsection.
+        * ``score_earned``: The total amount of points that the learner earned on the subsection.
+        * ``percent``: A float representing the overall grade for the course.
+        * ``module_id``: The ID of the subsection.
+        """
+        results = self.client.gradebook(course_id).get(user_contains=username).get('results')
+        for row in results:
+            if row.get('username') == username:
+                return row.get('section_breakdown')
+
+        raise HttpNotFoundError('No assessment grade record found for course={}, username={}'.format(
+            course_id,
+            username
+        ))
+
 
 class CertificatesApiClient(JwtLmsApiClient):
     """
@@ -463,23 +545,22 @@ class CertificatesApiClient(JwtLmsApiClient):
         return self.client.certificates(username).courses(course_id).get()
 
 
-def enroll_user_in_course_locally(user, course_id, mode):
+class NoAuthLMSClient(NoAuthenticationLmsApiClient):
     """
-    Enroll a user in a course, using local database methods.
-
-    This is only used in one place; in models.py. It is necessary
-    because the enrollment API can't enroll a user before it exists, and
-    the post_save signal for users results in a user not actually being
-    saved until after the API call is made.
-
-    In Django 1.9 and later, there's a transaction.on_commit hook that we can
-    use to create a callback. Once we're able to depend on having Django 1.9, we
-    can shift over to that, but for right now, we have to do it this way.
+    Class to build an LMS client to make calls to the LMS.
     """
-    if CourseEnrollment is None:
-        raise NotConnectedToOpenEdX("This package must be installed in an OpenEdX environment.")
 
-    CourseEnrollment.enroll(user, CourseKey.from_string(course_id), mode=mode, check_access=True)
+    API_BASE_URL = settings.LMS_INTERNAL_ROOT_URL
+    APPEND_SLASH = False
+
+    def get_health(self):
+        """
+        Retrieve health details for LMS service.
+
+        Returns:
+            dict: Response containing LMS service health.
+        """
+        return self.client.heartbeat.get()
 
 
 def parse_lms_api_datetime(datetime_string, datetime_format=LMS_API_DATETIME_FORMAT):
