@@ -2,7 +2,6 @@
 """
 Decorators for enterprise app.
 """
-from __future__ import absolute_import, unicode_literals
 
 import inspect
 import warnings
@@ -128,15 +127,17 @@ def enterprise_login_required(view):
 
         enterprise_uuid = kwargs['enterprise_uuid']
         enterprise_customer = get_enterprise_customer_or_404(enterprise_uuid)
+        query_params = request.GET
+        tpa_hint_param = query_params.get('tpa_hint')
 
         # Now verify if the user is logged in. If user is not logged in then
         # send the user to the login screen to sign in with an
         # Enterprise-linked IdP and the pipeline will get them back here.
-        if not request.user.is_authenticated():
+        if not request.user.is_authenticated:
             parsed_current_url = urlparse(request.get_full_path())
             parsed_query_string = parse_qs(parsed_current_url.query)
             parsed_query_string.update({
-                'tpa_hint': enterprise_customer.identity_provider,
+                'tpa_hint': enterprise_customer.get_tpa_hint(tpa_hint_param),
                 FRESH_LOGIN_PARAMETER: 'yes'
             })
             next_url = '{current_path}?{query_string}'.format(
@@ -199,7 +200,11 @@ def force_fresh_session(view):
             # log out and then come back here - the enterprise_login_required decorator will
             # then take effect prior to us arriving back here again.
             enterprise_customer = get_enterprise_customer_or_404(kwargs.get('enterprise_uuid'))
-            provider_id = enterprise_customer.identity_provider or ''
+            if not enterprise_customer.has_multiple_idps:
+                provider_id = enterprise_customer.identity_provider \
+                    if enterprise_customer.identity_provider else ''
+            else:
+                provider_id = ''
             sso_provider = get_identity_provider(provider_id)
             if sso_provider:
                 # Parse the current request full path, quote just the path portion,
