@@ -3,8 +3,6 @@
 Class for transmitting content metadata to Degreed.
 """
 
-from __future__ import absolute_import, unicode_literals
-
 from integrated_channels.degreed.client import DegreedAPIClient
 from integrated_channels.integrated_channel.transmitters.content_metadata import ContentMetadataTransmitter
 
@@ -18,7 +16,7 @@ class DegreedContentMetadataTransmitter(ContentMetadataTransmitter):
         """
         Use the ``DegreedAPIClient`` for content metadata transmission to Degreed.
         """
-        super(DegreedContentMetadataTransmitter, self).__init__(
+        super().__init__(
             enterprise_configuration=enterprise_configuration,
             client=client
         )
