@@ -3,8 +3,6 @@
 Class for transmitting learner data to Degreed.
 """
 
-from __future__ import absolute_import, unicode_literals
-
 from integrated_channels.degreed.client import DegreedAPIClient
 from integrated_channels.integrated_channel.transmitters.learner_data import LearnerTransmitter
 
@@ -19,7 +17,7 @@ class DegreedLearnerTransmitter(LearnerTransmitter):
         """
         By default, use the ``DegreedAPIClient`` for learner data transmission to Degreed.
         """
-        super(DegreedLearnerTransmitter, self).__init__(
+        super().__init__(
             enterprise_configuration=enterprise_configuration,
             client=client
         )
@@ -34,4 +32,4 @@ class DegreedLearnerTransmitter(LearnerTransmitter):
         kwargs['app_label'] = 'degreed'
         kwargs['model_name'] = 'DegreedLearnerDataTransmissionAudit'
         kwargs['remote_user_id'] = 'degreed_user_email'
-        super(DegreedLearnerTransmitter, self).transmit(payload, **kwargs)
+        super().transmit(payload, **kwargs)
