@@ -3,10 +3,8 @@
 Package for generic data exporters which serialize data to be transmitted to integrated channels.
 """
 
-from __future__ import absolute_import, unicode_literals
 
-
-class Exporter(object):
+class Exporter:
     """
     Interface for exporting data to be transmitted to an integrated channel.
 
@@ -28,7 +26,7 @@ class Exporter(object):
         self.enterprise_configuration = enterprise_configuration
         self.enterprise_customer = enterprise_configuration.enterprise_customer
 
-    def export(self):
+    def export(self, **kwargs):
         """
         Export (read: serialize) data to be used by a transmitter to transmit to an integrated channel.
         """
