@@ -3,28 +3,42 @@
 Enterprise Integrated Channel management commands.
 """
 
-from __future__ import absolute_import, unicode_literals
-
 from collections import OrderedDict
 
 from django.core.management.base import CommandError
 from django.utils.translation import ugettext as _
 
 from enterprise.models import EnterpriseCustomer
-from integrated_channels.sap_success_factors.models import SAPSuccessFactorsEnterpriseCustomerConfiguration
+from integrated_channels.blackboard.models import BlackboardEnterpriseCustomerConfiguration
+from integrated_channels.canvas.models import CanvasEnterpriseCustomerConfiguration
+from integrated_channels.cornerstone.models import CornerstoneEnterpriseCustomerConfiguration
 from integrated_channels.degreed.models import DegreedEnterpriseCustomerConfiguration
+from integrated_channels.moodle.models import MoodleEnterpriseCustomerConfiguration
+from integrated_channels.sap_success_factors.models import SAPSuccessFactorsEnterpriseCustomerConfiguration
 
 # Mapping between the channel code and the channel configuration class
 INTEGRATED_CHANNEL_CHOICES = OrderedDict([
     (integrated_channel_class.channel_code(), integrated_channel_class)
     for integrated_channel_class in (
-        SAPSuccessFactorsEnterpriseCustomerConfiguration,
+        BlackboardEnterpriseCustomerConfiguration,
+        CanvasEnterpriseCustomerConfiguration,
+        CornerstoneEnterpriseCustomerConfiguration,
         DegreedEnterpriseCustomerConfiguration,
+        MoodleEnterpriseCustomerConfiguration,
+        SAPSuccessFactorsEnterpriseCustomerConfiguration,
+    )
+])
+
+ASSESSMENT_LEVEL_REPORTING_INTEGRATED_CHANNEL_CHOICES = OrderedDict([
+    (integrated_channel_class.channel_code(), integrated_channel_class)
+    for integrated_channel_class in (
+        BlackboardEnterpriseCustomerConfiguration,
+        CanvasEnterpriseCustomerConfiguration,
     )
 ])
 
 
-class IntegratedChannelCommandMixin(object):
+class IntegratedChannelCommandMixin:
     """
     Contains common functionality for the IntegratedChannel management commands.
     """
@@ -48,7 +62,7 @@ class IntegratedChannelCommandMixin(object):
             metavar='INTEGRATED_CHANNEL',
             help=_('Transmit data to this IntegrateChannel. '
                    'Omit this option to transmit to all configured, active integrated channels.'),
-            choices=INTEGRATED_CHANNEL_CHOICES.keys(),
+            choices=list(INTEGRATED_CHANNEL_CHOICES.keys()),
         )
 
     def get_integrated_channels(self, options):
@@ -59,12 +73,17 @@ class IntegratedChannelCommandMixin(object):
 
         See ``add_arguments`` for the accepted options.
         """
-        channel_classes = self.get_channel_classes(options.get('channel'))
+        assessment_level_support = options.get('assessment_level_support', False)
+        channel_classes = self.get_channel_classes(
+            options.get('channel'),
+            assessment_level_support=assessment_level_support,
+        )
         filter_kwargs = {
             'active': True,
             'enterprise_customer__active': True,
         }
         enterprise_customer = self.get_enterprise_customer(options.get('enterprise_customer'))
+
         if enterprise_customer:
             filter_kwargs['enterprise_customer'] = enterprise_customer
 
@@ -83,12 +102,13 @@ class IntegratedChannelCommandMixin(object):
             return None
         try:
             return EnterpriseCustomer.active_customers.get(uuid=uuid)
-        except EnterpriseCustomer.DoesNotExist:
+        except EnterpriseCustomer.DoesNotExist as no_customer_exception:
             raise CommandError(
-                _('Enterprise customer {uuid} not found, or not active').format(uuid=uuid))
+                _('Enterprise customer {uuid} not found, or not active').format(uuid=uuid)
+            ) from no_customer_exception
 
     @staticmethod
-    def get_channel_classes(channel_code):
+    def get_channel_classes(channel_code, assessment_level_support=False):
         """
         Assemble a list of integrated channel classes to transmit to.
 
@@ -96,15 +116,26 @@ class IntegratedChannelCommandMixin(object):
 
         Otherwise, use all the available channel types.
         """
+        if assessment_level_support:
+            channel_choices = ASSESSMENT_LEVEL_REPORTING_INTEGRATED_CHANNEL_CHOICES
+        else:
+            channel_choices = INTEGRATED_CHANNEL_CHOICES
+
         if channel_code:
             # Channel code is case-insensitive
             channel_code = channel_code.upper()
 
-            if channel_code not in INTEGRATED_CHANNEL_CHOICES:
+            if channel_code not in channel_choices:
                 raise CommandError(_('Invalid integrated channel: {channel}').format(channel=channel_code))
 
-            channel_classes = [INTEGRATED_CHANNEL_CHOICES[channel_code]]
+            channel_classes = [channel_choices[channel_code]]
         else:
-            channel_classes = INTEGRATED_CHANNEL_CHOICES.values()
+            channel_classes = list(channel_choices.values())
 
         return channel_classes
+
+
+class IntegratedChannelCommandUtils(IntegratedChannelCommandMixin):
+    """
+    This is a wrapper class to avoid using mixin in methods
+    """
