@@ -3,26 +3,28 @@
 Database models for Enterprise Integrated Channel.
 """
 
-from __future__ import absolute_import, unicode_literals
-
 import json
 import logging
 
 from jsonfield.fields import JSONField
 
+from django.contrib import auth
+from django.core.exceptions import ValidationError
 from django.db import models
 from django.utils.encoding import python_2_unicode_compatible
 from django.utils.translation import ugettext_lazy as _
 
 from model_utils.models import TimeStampedModel
 
-from enterprise.models import EnterpriseCustomer
+from enterprise.models import EnterpriseCustomer, EnterpriseCustomerCatalog
 from integrated_channels.integrated_channel.exporters.content_metadata import ContentMetadataExporter
 from integrated_channels.integrated_channel.exporters.learner_data import LearnerExporter
 from integrated_channels.integrated_channel.transmitters.content_metadata import ContentMetadataTransmitter
 from integrated_channels.integrated_channel.transmitters.learner_data import LearnerTransmitter
+from integrated_channels.utils import convert_comma_separated_string_to_list
 
 LOGGER = logging.getLogger(__name__)
+User = auth.get_user_model()
 
 
 class EnterpriseCustomerPluginConfiguration(TimeStampedModel):
@@ -41,6 +43,7 @@ class EnterpriseCustomerPluginConfiguration(TimeStampedModel):
         blank=False,
         null=False,
         help_text=_("Enterprise Customer associated with the configuration."),
+        on_delete=models.deletion.CASCADE
     )
 
     active = models.BooleanField(
@@ -54,9 +57,57 @@ class EnterpriseCustomerPluginConfiguration(TimeStampedModel):
         help_text=_("The maximum number of data items to transmit to the integrated channel with each request.")
     )
 
+    channel_worker_username = models.CharField(
+        max_length=255,
+        blank=True,
+        null=True,
+        help_text=_("Enterprise channel worker username to get JWT tokens for authenticating LMS APIs."),
+    )
+    catalogs_to_transmit = models.TextField(
+        blank=True,
+        null=True,
+        help_text=_("A comma-separated list of catalog UUIDs to transmit."),
+    )
+
     class Meta:
         abstract = True
 
+    def clean(self):
+        invalid_uuids = []
+        for uuid in convert_comma_separated_string_to_list(self.catalogs_to_transmit):
+            try:
+                EnterpriseCustomerCatalog.objects.get(uuid=uuid, enterprise_customer=self.enterprise_customer)
+            except (EnterpriseCustomerCatalog.DoesNotExist, ValidationError):
+                invalid_uuids.append(str(uuid))
+        if invalid_uuids:
+            raise ValidationError(
+                {
+                    'catalogs_to_transmit': [
+                        "These are the invalid uuids: {invalid_uuids}".format(invalid_uuids=invalid_uuids)
+                    ]
+                }
+            )
+
+    @property
+    def channel_worker_user(self):
+        """
+        default worker username for channel
+        """
+        worker_username = self.channel_worker_username if self.channel_worker_username else 'enterprise_channel_worker'
+        return User.objects.filter(username=worker_username).first()
+
+    @property
+    def customer_catalogs_to_transmit(self):
+        """
+        Return the list of EnterpriseCustomerCatalog objects.
+        """
+        catalogs_list = []
+        if self.catalogs_to_transmit:
+            catalogs_list = EnterpriseCustomerCatalog.objects.filter(
+                uuid__in=convert_comma_separated_string_to_list(self.catalogs_to_transmit)
+            )
+        return catalogs_list
+
     @staticmethod
     def channel_code():
         """
@@ -97,6 +148,14 @@ class EnterpriseCustomerPluginConfiguration(TimeStampedModel):
         transmitter = self.get_learner_data_transmitter()
         transmitter.transmit(exporter)
 
+    def transmit_single_learner_data(self, **kwargs):
+        """
+        Iterate over single learner data record and transmit it to the integrated channel.
+        """
+        exporter = self.get_learner_data_exporter(self.channel_worker_user)
+        transmitter = self.get_learner_data_transmitter()
+        transmitter.transmit(exporter, **kwargs)
+
     def transmit_content_metadata(self, user):
         """
         Transmit content metadata to integrated channel.
@@ -105,14 +164,32 @@ class EnterpriseCustomerPluginConfiguration(TimeStampedModel):
         transmitter = self.get_content_metadata_transmitter()
         transmitter.transmit(exporter.export())
 
+    def transmit_single_subsection_learner_data(self, **kwargs):
+        """
+        Transmit a single subsection learner data record to the integrated channel.
+        """
+        exporter = self.get_learner_data_exporter(self.channel_worker_user)
+        transmitter = self.get_learner_data_transmitter()
+        transmitter.single_learner_assessment_grade_transmit(exporter, **kwargs)
+
+    def transmit_subsection_learner_data(self, user):
+        """
+        Iterate over each assessment learner data record and transmit them to the integrated channel.
+        """
+        exporter = self.get_learner_data_exporter(user)
+        transmitter = self.get_learner_data_transmitter()
+        transmitter.assessment_level_transmit(exporter)
+
 
 @python_2_unicode_compatible
 class LearnerDataTransmissionAudit(models.Model):
     """
     The payload we send to an integrated channel  at a given point in time for an enterprise course enrollment.
+
+    .. no_pii:
     """
 
-    enterprise_course_enrollment_id = models.PositiveIntegerField(blank=False, null=False)
+    enterprise_course_enrollment_id = models.PositiveIntegerField(blank=False, null=False, db_index=True)
     course_id = models.CharField(max_length=255, blank=False, null=False)
     course_completed = models.BooleanField(default=True)
     completed_timestamp = models.BigIntegerField()
@@ -122,6 +199,8 @@ class LearnerDataTransmissionAudit(models.Model):
     error_message = models.TextField(blank=True)
     created = models.DateTimeField(auto_now_add=True)
 
+    subsection_id = models.CharField(max_length=255, blank=True, null=True)
+
     class Meta:
         app_label = 'integrated_channel'
 
@@ -182,14 +261,16 @@ class ContentMetadataItemTransmission(TimeStampedModel):
     This model can be queried to find the content metadata items that have been
     transmitted to an integrated channel. It is used to synchronize the content
     metadata items available in an enterprise's catalog with the integrated channel.
+
+    .. no_pii:
     """
 
-    enterprise_customer = models.ForeignKey(EnterpriseCustomer)
+    enterprise_customer = models.ForeignKey(EnterpriseCustomer, on_delete=models.CASCADE)
     integrated_channel_code = models.CharField(max_length=30)
     content_id = models.CharField(max_length=255)
     channel_metadata = JSONField()
 
-    class Meta(object):
+    class Meta:
         unique_together = ('enterprise_customer', 'integrated_channel_code', 'content_id')
 
     def __str__(self):
