@@ -2,16 +2,18 @@
 """
 Django admin integration for configuring sap_success_factors app to communicate with SAP SuccessFactors systems.
 """
-from __future__ import absolute_import, unicode_literals
 
-from django.contrib import admin
 from config_models.admin import ConfigurationModelAdmin
 from requests import RequestException
 
-from integrated_channels.sap_success_factors.models import (
-    SAPSuccessFactorsEnterpriseCustomerConfiguration, SAPSuccessFactorsGlobalConfiguration
-)
+from django.contrib import admin
+
+from integrated_channels.exceptions import ClientError
 from integrated_channels.sap_success_factors.client import SAPSuccessFactorsAPIClient
+from integrated_channels.sap_success_factors.models import (
+    SAPSuccessFactorsEnterpriseCustomerConfiguration,
+    SAPSuccessFactorsGlobalConfiguration,
+)
 
 
 @admin.register(SAPSuccessFactorsGlobalConfiguration)
@@ -24,9 +26,10 @@ class SAPSuccessFactorsGlobalConfigurationAdmin(ConfigurationModelAdmin):
         "course_api_path",
         "oauth_api_path",
         "provider_id",
+        "search_student_api_path",
     )
 
-    class Meta(object):
+    class Meta:
         model = SAPSuccessFactorsGlobalConfiguration
 
 
@@ -45,15 +48,16 @@ class SAPSuccessFactorsEnterpriseCustomerConfigurationAdmin(admin.ModelAdmin):
         'sapsf_user_id',
         'user_type',
         'has_access_token',
+        'transmit_total_hours',
         'transmission_chunk_size',
         'additional_locales',
+        'catalogs_to_transmit',
     )
 
     list_display = (
         'enterprise_customer_name',
         'active',
         'sapsf_base_url',
-        'has_access_token',
         'modified',
     )
     ordering = ('enterprise_customer__name',)
@@ -65,7 +69,7 @@ class SAPSuccessFactorsEnterpriseCustomerConfigurationAdmin(admin.ModelAdmin):
     list_filter = ('active',)
     search_fields = ('enterprise_customer__name',)
 
-    class Meta(object):
+    class Meta:
         model = SAPSuccessFactorsEnterpriseCustomerConfiguration
 
     def enterprise_customer_name(self, obj):
@@ -97,7 +101,7 @@ class SAPSuccessFactorsEnterpriseCustomerConfigurationAdmin(admin.ModelAdmin):
                 obj.sapsf_user_id,
                 obj.user_type
             )
-        except (RequestException, ValueError):
+        except (RequestException, ClientError):
             return False
         return bool(access_token and expires_at)
 
