@@ -2,18 +2,21 @@
 """
 Client for connecting to SAP SuccessFactors.
 """
-from __future__ import absolute_import, unicode_literals
 
 import datetime
+import logging
 import time
 
 import requests
+from requests.exceptions import ConnectionError, Timeout  # pylint: disable=redefined-builtin
 
 from django.apps import apps
 
 from integrated_channels.exceptions import ClientError
 from integrated_channels.integrated_channel.client import IntegratedChannelApiClient
 
+LOGGER = logging.getLogger(__name__)
+
 
 class SAPSuccessFactorsAPIClient(IntegratedChannelApiClient):  # pylint: disable=abstract-method
     """
@@ -41,7 +44,7 @@ class SAPSuccessFactorsAPIClient(IntegratedChannelApiClient):  # pylint: disable
             tuple: Tuple containing access token string and expiration datetime.
         Raises:
             HTTPError: If we received a failure response code from SAP SuccessFactors.
-            RequestException: If an unexpected response format was received that we could not parse.
+            ClientError: If an unexpected response format was received that we could not parse.
         """
         SAPSuccessFactorsGlobalConfiguration = apps.get_model(  # pylint: disable=invalid-name
             'sap_success_factors',
@@ -65,12 +68,19 @@ class SAPSuccessFactorsAPIClient(IntegratedChannelApiClient):  # pylint: disable
             headers={'content-type': 'application/json'}
         )
 
-        response.raise_for_status()
-        data = response.json()
         try:
+            data = response.json()
             return data['access_token'], datetime.datetime.utcfromtimestamp(data['expires_in'] + int(time.time()))
-        except KeyError:
-            raise requests.RequestException(response=response)
+        except (KeyError, TypeError, ValueError) as error:
+            LOGGER.error(
+                'SAP SF OAuth2 POST response is of invalid format. User: [%s], Company: [%s],'
+                ' Error: [%s], Response: [%s]',
+                str(user_id),
+                str(company_id),
+                str(error),
+                str(response)
+            )
+            raise ClientError(response, response.status_code) from error
 
     def __init__(self, enterprise_configuration):
         """
@@ -80,30 +90,40 @@ class SAPSuccessFactorsAPIClient(IntegratedChannelApiClient):  # pylint: disable
             enterprise_configuration (SAPSuccessFactorsEnterpriseCustomerConfiguration): An enterprise customers's
             configuration model for connecting with SAP SuccessFactors
         """
-        super(SAPSuccessFactorsAPIClient, self).__init__(enterprise_configuration)
+        super().__init__(enterprise_configuration)
         self.global_sap_config = apps.get_model('sap_success_factors', 'SAPSuccessFactorsGlobalConfiguration').current()
-        self._create_session()
+        self.session = None
+        self.expires_at = None
 
     def _create_session(self):
         """
         Instantiate a new session object for use in connecting with SAP SuccessFactors
         """
-        session = requests.Session()
-        session.timeout = self.SESSION_TIMEOUT
+        now = datetime.datetime.utcnow()
+        if self.session is None or self.expires_at is None or now >= self.expires_at:
+            # Create a new session with a valid token
+            if self.session:
+                self.session.close()
 
-        oauth_access_token, expires_at = SAPSuccessFactorsAPIClient.get_oauth_access_token(
-            self.enterprise_configuration.sapsf_base_url,
-            self.enterprise_configuration.key,
-            self.enterprise_configuration.secret,
-            self.enterprise_configuration.sapsf_company_id,
-            self.enterprise_configuration.sapsf_user_id,
-            self.enterprise_configuration.user_type
-        )
+            oauth_access_token, expires_at = SAPSuccessFactorsAPIClient.get_oauth_access_token(
+                self.enterprise_configuration.sapsf_base_url,
+                self.enterprise_configuration.key,
+                self.enterprise_configuration.secret,
+                self.enterprise_configuration.sapsf_company_id,
+                self.enterprise_configuration.sapsf_user_id,
+                self.enterprise_configuration.user_type
+            )
+            session = requests.Session()
+            session.timeout = self.SESSION_TIMEOUT
+            session.headers['Authorization'] = 'Bearer {}'.format(oauth_access_token)
+            session.headers['content-type'] = 'application/json'
+            self.session = session
+            self.expires_at = expires_at
 
-        session.headers['Authorization'] = 'Bearer {}'.format(oauth_access_token)
-        session.headers['content-type'] = 'application/json'
-        self.session = session
-        self.expires_at = expires_at
+    def create_assessment_reporting(self, user_id, payload):
+        """
+        Not implemented yet
+        """
 
     def create_course_completion(self, user_id, payload):
         """
@@ -177,7 +197,7 @@ class SAPSuccessFactorsAPIClient(IntegratedChannelApiClient):  # pylint: disable
                     error=exc.__class__.__name__,
                     message=str(exc)
                 )
-            )
+            ) from exc
 
         if status_code >= 400:
             raise ClientError(
@@ -218,6 +238,13 @@ class SAPSuccessFactorsAPIClient(IntegratedChannelApiClient):  # pylint: disable
             }
         )
 
+        if response.status_code >= 400:
+            raise ClientError(
+                'SAPSuccessFactorsAPIClient request failed with status {status_code}: {message}'.format(
+                    status_code=response.status_code,
+                    message=response.text
+                )
+            )
         return response.status_code, response.text
 
     def _call_post_with_session(self, url, payload):
@@ -228,10 +255,115 @@ class SAPSuccessFactorsAPIClient(IntegratedChannelApiClient):  # pylint: disable
             url (str): The url to post to.
             payload (str): The json encoded payload to post.
         """
-        now = datetime.datetime.utcnow()
-        if now >= self.expires_at:
-            # Create a new session with a valid token
-            self.session.close()
-            self._create_session()
+        self._create_session()
         response = self.session.post(url, data=payload)
         return response.status_code, response.text
+
+    def get_inactive_sap_learners(self):
+        """
+        Make a GET request using the session object to a SuccessFactors endpoint for inactive learners.
+
+        Example:
+            sap_search_student_url: "/learning/odatav4/searchStudent/v1/Students?
+                $filter=criteria/isActive eq False&$select=studentID"
+
+            SAP API response: {
+                u'@odata.metadataEtag': u'W/"17090d86-20fa-49c8-8de0-de1d308c8b55"',
+                u'value': [
+                    {
+                        u'studentID': u'admint6',
+                    },
+                    {
+                        u'studentID': u'adminsap1',
+                    }
+                ]
+            }
+
+        Returns: List of inactive learners
+        [
+            {
+                u'studentID': u'admint6'
+            },
+            {
+                u'studentID': u'adminsap1'
+            }
+        ]
+        """
+        self._create_session()
+        sap_search_student_url = '{sapsf_base_url}/{search_students_path}?$filter={search_filter}'.format(
+            sapsf_base_url=self.enterprise_configuration.sapsf_base_url.rstrip('/'),
+            search_students_path=self.global_sap_config.search_student_api_path.rstrip('/'),
+            search_filter='criteria/isActive eq False&$select=studentID',
+        )
+        all_inactive_learners = self._call_search_students_recursively(
+            sap_search_student_url,
+            all_inactive_learners=[],
+            page_size=500,
+            start_at=0
+        )
+        return all_inactive_learners
+
+    def _call_search_students_recursively(self, sap_search_student_url, all_inactive_learners, page_size, start_at):
+        """
+        Make recursive GET calls to traverse the paginated API response for search students.
+        """
+        search_student_paginated_url = '{sap_search_student_url}&{pagination_criterion}'.format(
+            sap_search_student_url=sap_search_student_url,
+            pagination_criterion='$count=true&$top={page_size}&$skip={start_at}'.format(
+                page_size=page_size,
+                start_at=start_at,
+            ),
+        )
+        try:
+            response = self.session.get(search_student_paginated_url)
+            sap_inactive_learners = response.json()
+        except ValueError as error:
+            raise ClientError(response, response.status_code) from error
+        except (ConnectionError, Timeout):
+            LOGGER.warning(
+                'Unable to fetch inactive learners from SAP searchStudent API with url '
+                '"{%s}".', search_student_paginated_url,
+            )
+            return None
+
+        if 'error' in sap_inactive_learners:
+            try:
+                LOGGER.warning(
+                    'SAP searchStudent API for customer %s and base url %s returned response with '
+                    'error message "%s" and with error code "%s".',
+                    self.enterprise_configuration.enterprise_customer.name,
+                    self.enterprise_configuration.sapsf_base_url,
+                    sap_inactive_learners['error'].get('message'),
+                    sap_inactive_learners['error'].get('code'),
+                )
+            except AttributeError:
+                LOGGER.warning(
+                    'SAP searchStudent API for customer %s and base url %s returned response with '
+                    'error message "%s" and with error code "%s".',
+                    self.enterprise_configuration.enterprise_customer.name,
+                    self.enterprise_configuration.sapsf_base_url,
+                    sap_inactive_learners['error'],
+                    response.status_code,
+                )
+            return None
+
+        new_page_start_at = page_size + start_at
+        total_inactive_learners = sap_inactive_learners['@odata.count']
+        inactive_learners_on_page = sap_inactive_learners['value']
+        LOGGER.info(
+            'SAP SF searchStudent API returned [%d] inactive learners of total [%d] starting from [%d] for '
+            'enterprise customer [%s]',
+            len(inactive_learners_on_page), total_inactive_learners, start_at,
+            self.enterprise_configuration.enterprise_customer.name
+        )
+
+        all_inactive_learners += inactive_learners_on_page
+        if total_inactive_learners > new_page_start_at:
+            return self._call_search_students_recursively(
+                sap_search_student_url,
+                all_inactive_learners,
+                page_size=page_size,
+                start_at=new_page_start_at,
+            )
+
+        return all_inactive_learners
