@@ -3,8 +3,6 @@
 Database models for Enterprise Integrated Channel SAP SuccessFactors.
 """
 
-from __future__ import absolute_import, unicode_literals
-
 import json
 from logging import getLogger
 
@@ -14,26 +12,35 @@ from django.db import models
 from django.utils.encoding import python_2_unicode_compatible
 from django.utils.translation import ugettext_lazy as _
 
+from integrated_channels.exceptions import ClientError
 from integrated_channels.integrated_channel.models import EnterpriseCustomerPluginConfiguration
 from integrated_channels.sap_success_factors.exporters.content_metadata import SapSuccessFactorsContentMetadataExporter
-from integrated_channels.sap_success_factors.exporters.learner_data import SapSuccessFactorsLearnerExporter
+from integrated_channels.sap_success_factors.exporters.learner_data import (
+    SapSuccessFactorsLearnerExporter,
+    SapSuccessFactorsLearnerManger,
+)
 from integrated_channels.sap_success_factors.transmitters.content_metadata import (
     SapSuccessFactorsContentMetadataTransmitter,
 )
 from integrated_channels.sap_success_factors.transmitters.learner_data import SapSuccessFactorsLearnerTransmitter
+from integrated_channels.utils import convert_comma_separated_string_to_list
 
 LOGGER = getLogger(__name__)
 
 
+# pylint: disable=feature-toggle-needs-doc
 @python_2_unicode_compatible
 class SAPSuccessFactorsGlobalConfiguration(ConfigurationModel):
     """
     The global configuration for integrating with SuccessFactors.
+
+    .. no_pii:
     """
 
     completion_status_api_path = models.CharField(max_length=255)
     course_api_path = models.CharField(max_length=255)
     oauth_api_path = models.CharField(max_length=255)
+    search_student_api_path = models.CharField(max_length=255)
     provider_id = models.CharField(max_length=100, default='EDX')
 
     class Meta:
@@ -56,6 +63,8 @@ class SAPSuccessFactorsGlobalConfiguration(ConfigurationModel):
 class SAPSuccessFactorsEnterpriseCustomerConfiguration(EnterpriseCustomerPluginConfiguration):
     """
     The Enterprise-specific configuration we need for integrating with SuccessFactors.
+
+    .. no_pii:
     """
 
     USER_TYPE_USER = 'user'
@@ -102,6 +111,12 @@ class SAPSuccessFactorsEnterpriseCustomerConfiguration(EnterpriseCustomerPluginC
         verbose_name="Additional Locales",
         help_text=_("A comma-separated list of additional locales.")
     )
+    show_course_price = models.BooleanField(default=False)
+    transmit_total_hours = models.BooleanField(
+        default=False,
+        verbose_name=_("Transmit Total Hours"),
+        help_text=_("Include totalHours in the transmitted completion data")
+    )
 
     def get_locales(self, default_locale=None):
         """
@@ -121,7 +136,7 @@ class SAPSuccessFactorsEnterpriseCustomerConfiguration(EnterpriseCustomerPluginC
             locales.append(default_locale)
 
         return set(
-            locales + [locale.strip() for locale in self.additional_locales.split(",") if locale.strip()]
+            locales + convert_comma_separated_string_to_list(self.additional_locales)
         )
 
     class Meta:
@@ -179,19 +194,44 @@ class SAPSuccessFactorsEnterpriseCustomerConfiguration(EnterpriseCustomerPluginC
         """
         return SapSuccessFactorsContentMetadataExporter(user, self)
 
+    def get_learner_manger(self):
+        """
+        Return a ``SapSuccessFactorsLearnerManger`` instance.
+        """
+        return SapSuccessFactorsLearnerManger(self)
+
+    def unlink_inactive_learners(self):
+        """
+        Unlink inactive SAP learners form their related enterprises
+        """
+        sap_learner_manager = self.get_learner_manger()
+        try:
+            sap_learner_manager.unlink_learners()
+        except ClientError as exc:
+            LOGGER.exception(
+                'Failed to unlink learners for integrated channel [%s] [%s] \nError: [%s]',
+                self.enterprise_customer.name,
+                self.channel_code(),
+                str(exc)
+            )
+
 
 @python_2_unicode_compatible
 class SapSuccessFactorsLearnerDataTransmissionAudit(models.Model):
     """
     The payload we sent to SuccessFactors at a given point in time for an enterprise course enrollment.
+
+    .. no_pii:
     """
 
     sapsf_user_id = models.CharField(max_length=255, blank=False, null=False)
-    enterprise_course_enrollment_id = models.PositiveIntegerField(blank=False, null=False)
+    enterprise_course_enrollment_id = models.PositiveIntegerField(blank=False, null=False, db_index=True)
     course_id = models.CharField(max_length=255, blank=False, null=False)
     course_completed = models.BooleanField(default=True)
     instructor_name = models.CharField(max_length=255, blank=True)
     grade = models.CharField(max_length=100, blank=False, null=False)
+    total_hours = models.FloatField(null=True, blank=True)
+    credit_hours = models.FloatField(null=True, blank=True)
 
     # We send a UNIX timestamp to SAPSF.
     completed_timestamp = models.BigIntegerField()
@@ -253,4 +293,6 @@ class SapSuccessFactorsLearnerDataTransmissionAudit(models.Model):
             courseCompleted="true" if self.course_completed else "false",
             completedTimestamp=self.completed_timestamp,
             grade=self.grade,
+            totalHours=self.total_hours,
+            creditHours=self.credit_hours,
         )
