@@ -3,10 +3,10 @@
 Class for transmitting learner data to SuccessFactors.
 """
 
-from __future__ import absolute_import, unicode_literals
-
 import logging
 
+import six
+
 from enterprise.models import EnterpriseCustomerUser
 from integrated_channels.integrated_channel.transmitters.learner_data import LearnerTransmitter
 from integrated_channels.sap_success_factors.client import SAPSuccessFactorsAPIClient
@@ -24,7 +24,7 @@ class SapSuccessFactorsLearnerTransmitter(LearnerTransmitter):
         """
         By default, use the ``SAPSuccessFactorsAPIClient`` for learner data transmission to SAPSF.
         """
-        super(SapSuccessFactorsLearnerTransmitter, self).__init__(
+        super().__init__(
             enterprise_configuration=enterprise_configuration,
             client=client
         )
@@ -39,16 +39,16 @@ class SapSuccessFactorsLearnerTransmitter(LearnerTransmitter):
         kwargs['app_label'] = 'sap_success_factors'
         kwargs['model_name'] = 'SapSuccessFactorsLearnerDataTransmissionAudit'
         kwargs['remote_user_id'] = 'sapsf_user_id'
-        super(SapSuccessFactorsLearnerTransmitter, self).transmit(payload, **kwargs)
+        super().transmit(payload, **kwargs)
 
-    def handle_transmission_error(self, learner_data, request_exception):
+    def handle_transmission_error(self, learner_data, client_exception):
         """Handle the case where the employee on SAPSF's side is marked as inactive."""
         try:
-            sys_msg = request_exception.response.content
+            sys_msg = six.text_type(client_exception.message)
         except AttributeError:
             pass
         else:
-            if 'user account is inactive' in sys_msg:
+            if sys_msg and 'user account is inactive' in sys_msg:
                 ecu = EnterpriseCustomerUser.objects.get(
                     enterprise_enrollments__id=learner_data.enterprise_course_enrollment_id)
                 ecu.active = False
@@ -59,4 +59,4 @@ class SapSuccessFactorsLearnerTransmitter(LearnerTransmitter):
                     ecu.username, ecu.user_id, ecu.user_email, ecu.enterprise_customer
                 )
                 return
-        super(SapSuccessFactorsLearnerTransmitter, self).handle_transmission_error(learner_data, request_exception)
+        super().handle_transmission_error(learner_data, client_exception)
